package de.espirit.firstspirit.opt.example.universal.dataaccess;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.agency.Image;
import de.espirit.firstspirit.agency.ImageAgent;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessPlugin;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSessionBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.DataAccessAspectMap;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.DataAccessAspectType;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.ReportItemsProviding;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Reporting;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.reporting.TextBlocksReportItemsProvidingDataAccessPluginAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.reporting.TextBlocksReportingDataAccessPluginAspect;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.ImageIcon;


/**
 * Example implementation of {@link DataAccessPlugin DataAccessPlugin<D>}, which acts as an external data source for
 * FS_INDEX input components and is also shown as a report in SiteArchitect and ContentCreator.
 */
public class TextBlocksDataAccessPlugin implements DataAccessPlugin<TextBlocksObject> {

	@NotNull
	private final DataAccessAspectMap _aspects;
	private BaseContext _context;


	public TextBlocksDataAccessPlugin() {
		// Initialize a DataAccessAspectMap object in which aspect objects may be registered.
		_aspects = new DataAccessAspectMap();
	}


	/**
	 * Provides the requested aspect object, if available.
	 *
	 * @param type The type of aspect.
	 * @param <A> The requested aspect interface.
	 * @return An implementation of {@code <A>}, or {@code null} if no aspect of that type exists in the aspect map.
	 */
	@Nullable
	@Override
	public <A> A getAspect(@NotNull final DataAccessAspectType<A> type) {
		return _aspects.get(type);
	}


	/**
	 * Provides the name of this DataAccessPlugin implementation. This name will be used in the user interfaces for
	 * FS_INDEX input components and reports based on this class.
	 *
	 * @return This data access plug-in's label.
	 */
	@NotNull
	@Override
	public String getLabel() {
		return "Text Blocks (DataAccessPlugin example)";
	}


	/**
	 * Provides an icon for use with the Text Blocks data access plug-in.
	 *
	 * @return A client-specific {@link Image} that contains data about the icon image.
	 */
	@Nullable
	@Override
	public Image<?> getIcon() {
		final ImageAgent imageAgent = _context.requireSpecialist(ImageAgent.TYPE);

		// Create an Image object based on the FirstSpirit client's requirements.
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// For use in ContentCreator, image files need to be rolled out to the project-specific ContentCreator web application's
			// directory. URLs are relative to that directory.
			return imageAgent.getImageFromUrl("webclient_examples/media/dataaccess/textblocks/reportbutton_grey_dark.png");
		} else {
			// For SiteArchitect, the image file is loaded as a resource with a path relative to the class TextBlocksDataAccessPlugin.
			return imageAgent.getImageFromIcon(new ImageIcon(TextBlocksDataAccessPlugin.class.getResource("resource/icon/reportbutton_grey_dark.png")));
		}
	}


	/**
	 * Instantiates a {@link TextBlocksDataAccessSessionBuilder} object which handles configuration requirements and
	 * provides a {@link TextBlocksDataAccessSession} object for handling of text block data.
	 *
	 * @return A {@link TextBlocksDataAccessSessionBuilder}.
	 */
	@NotNull
	@Override
	public DataAccessSessionBuilder<TextBlocksObject> createSessionBuilder() {
		return new TextBlocksDataAccessSessionBuilder();
	}


	/**
	 * Performs basic set-up, including registration of appropriate aspect objects.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;

		// Add reporting-specific aspects to the aspect map.

		// Advertise this DataAccessPlugin as report-enabled.
		_aspects.put(Reporting.TYPE, new TextBlocksReportingDataAccessPluginAspect(_context));

		// Advertise that individual entries in this report support clickable items for interaction.
		_aspects.put(ReportItemsProviding.TYPE, new TextBlocksReportItemsProvidingDataAccessPluginAspect());
	}


	/**
	 * Performs any necessary clean-up just before the DataAccessPlugin object is discarded.
	 */
	@Override
	public void tearDown() {
		// No need to tear down anything.
	}
}
