package de.espirit.firstspirit.opt.example.universal.dataaccess;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.DataSnippetProvider;
import de.espirit.firstspirit.client.plugin.dataaccess.DataStreamBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.DataTemplating;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Identifying;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.JsonSupporting;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.SessionAspectMap;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.SessionAspectType;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.ValueIndexing;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.transfer.TransferHandling;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.transfer.TransferSupplying;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksIdentifyingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksJsonSupportingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksTransferHandlingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksTransferSupplyingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksValueIndexingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.reporting.TextBlocksDataTemplatingDataAccessSessionAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.provider.TextBlocksObjectProvider;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.NoSuchElementException;


/**
 * This class represents the core of the data access plug-in. It provides access to a data stream object and
 * defines one or more aspects which implement functionality that allow the plug-in's data to integrate into FirstSpirit.
 */
public class TextBlocksDataAccessSession implements DataAccessSession<TextBlocksObject> {

	private final BaseContext _context;

	private final SessionAspectMap _aspects;
	private final TextBlocksObjectProvider _textBlocksObjectProvider;


	/**
	 * Initializes a {@link SessionAspectMap} object and registers appropriate aspect objects in that map.
	 */
	public TextBlocksDataAccessSession(@NotNull final BaseContext context) {
		_context = context;
		_aspects = new SessionAspectMap();
		_textBlocksObjectProvider = new TextBlocksObjectProvider();

		// Add session-specific aspects to the aspect map.

		// Advertise that the individual TextBlocksObject objects support identifying features.
		_aspects.put(Identifying.TYPE, new TextBlocksIdentifyingDataAccessSessionAspect(_textBlocksObjectProvider));

		// Advertise that this plug-in provides detail views for use in ContentCreator. The data provided by this aspect
		// will be rendered in fly-outs shown as the mouse cursor hovers over a report entry.
		_aspects.put(DataTemplating.TYPE, new TextBlocksDataTemplatingDataAccessSessionAspect(_context));

		// Advertise support for value indexing, allowing entries referenced in an FS_INDEX input component to be
		// indexed for search purposes.
		_aspects.put(ValueIndexing.TYPE, new TextBlocksValueIndexingDataAccessSessionAspect(_textBlocksObjectProvider));

		// Advertise that this plug-in can supply transfer data for individual text block entries.
		_aspects.put(TransferSupplying.TYPE, new TextBlocksTransferSupplyingDataAccessSessionAspect(_context));

		// Advertise that this plug-in can handle transfer data from outside sources.
		_aspects.put(TransferHandling.TYPE, new TextBlocksTransferHandlingDataAccessSessionAspect(_context));

		// Advertise that this plug-in supports rendering as json
		_aspects.put(JsonSupporting.TYPE, new TextBlocksJsonSupportingDataAccessSessionAspect());
	}


	/**
	 * Provides the requested aspect object, if available.
	 *
	 * @param type The type of aspect.
	 * @param <A> The requested aspect interface.
	 * @return An implementation of {@code <A>}, or {@code null} if no aspect of that type exists in the aspect map.
	 */
	@Nullable
	@Override
	public <A> A getAspect(@NotNull final SessionAspectType<A> type) {
		return _aspects.get(type);
	}


	/**
	 * Provides a single {@link TextBlocksObject} based on a unique identifier.
	 *
	 * @param identifier The identifier to look up.
	 * @return A {@link TextBlocksObject}.
	 * @throws {@link NoSuchElementException} if no text block matches the given identifier.
	 */
	@NotNull
	@Override
	public TextBlocksObject getData(@NotNull final String identifier) throws NoSuchElementException {
		return _textBlocksObjectProvider.getSingleTextBlock(identifier).orElseThrow(() -> new NoSuchElementException("Could not find text blocks object with identifier '" + identifier + "'."));
	}


	/**
	 * Provides a list of {@link TextBlocksObject}s based on a list of unique identifiers.
	 *
	 * NOTE that the {@link List} we return MUST match the size of the {@link Collection} {@code identifiers}, and its entry order MUST correspond
	 * to the order of identifiers in {@code identifiers}!
	 *
	 * @param identifiers The identifiers to fetch data objects for.
	 * @return A {@link List} of {@link TextBlocksObject}s whose size matches that of {@code identifiers} and whose order of entries associates with that of {@code identifiers}.
	 */
	@NotNull
	@Override
	public List<TextBlocksObject> getData(@NotNull final Collection<String> identifiers) {
		final List<TextBlocksObject> textBlocksObjects = new ArrayList<TextBlocksObject>();
		// Walk through the Collection of identifiers and assemble a List, observing the size and entry order of the Collection object.
		for (final String identifier : identifiers) {
			try {
				final TextBlocksObject textBlocksObject = getData(identifier);
				textBlocksObjects.add(textBlocksObject);
			} catch (final NoSuchElementException e) {
				// The List we return MUST observe the size and order of the Collection this method has been given as a parameter.
				// So, if we cannot identify a text block object for a given identifier in the Collection, we must insert a null value in the List.
				textBlocksObjects.add(null);
			}
		}
		return textBlocksObjects;
	}


	/**
	 * Provides a unique identifier for the given {@link TextBlocksObject}.
	 *
	 * @param object The data object to provide an identifer for.
	 * @return A {@link String} identifier.
	 * @throws {@link NoSuchElementException} (not used here, since a {@link TextBlocksObject} must have an ID which we're generating the string identifier from)
	 */
	@NotNull
	@Override
	public String getIdentifier(@NotNull final TextBlocksObject object) throws NoSuchElementException {
		return String.valueOf(object.getId());
	}


	/**
	 * Instantiates a {@link TextBlocksDataSnippetProvider} object which will render data about individual {@link TextBlocksObject}s
	 * for snippet display in Text Blocks reports and FS_INDEX input components.
	 *
	 * @return A {@link DataSnippetProvider DataSnippetProvider<D>}.
	 */
	@NotNull
	@Override
	public DataSnippetProvider<TextBlocksObject> createDataSnippetProvider() {
		return new TextBlocksDataSnippetProvider(_context);
	}


	/**
	 * Instantiates a {@link TextBlocksDataStreamBuilder} object which configures and provides functionality centered
	 * around providing {@link TextBlocksObject}s from some external source.
	 *
	 * @return A {@link DataStreamBuilder DataStreamBuilder<D>}.
	 */
	@NotNull
	@Override
	public DataStreamBuilder<TextBlocksObject> createDataStreamBuilder() {
		return new TextBlocksDataStreamBuilder(_textBlocksObjectProvider);
	}
}
