package de.espirit.firstspirit.opt.example.universal.dataaccess;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.agency.Image;
import de.espirit.firstspirit.agency.ImageAgent;
import de.espirit.firstspirit.client.plugin.dataaccess.DataSnippetProvider;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.ImageIcon;
import java.net.URL;


/**
 * This class is used to generate data (i.e. icon and thumbnail as well as header and extract texts) which are shown
 * in snippet representations of individual text block entries in FS_INDEX input components and reports based on this plug-in.
 */
public class TextBlocksDataSnippetProvider implements DataSnippetProvider<TextBlocksObject> {

	private final BaseContext _context;


	public TextBlocksDataSnippetProvider(@NotNull final BaseContext context) {
		_context = context;
	}


	/**
	 * Provides an icon for the text block.
	 *
	 * @param object The object that is to be rendered in report lists.
	 * @return A client-specific {@link Image} that contains data about the icon image.
	 */
	@Nullable
	@Override
	public Image<?> getIcon(@NotNull final TextBlocksObject object) {
		final ImageAgent imageAgent = _context.requireSpecialist(ImageAgent.TYPE);

		final String fileName = "reportbutton_grey_dark.png";

		// Create an Image object based on the FirstSpirit client's requirements.
		if (_context.is(BaseContext.Env.WEBEDIT) || _context.is(BaseContext.Env.PREVIEW) || _context.is(BaseContext.Env.HEADLESS)) {
			// For use in ContentCreator, image files need to be rolled out to the project-specific ContentCreator web application's
			// directory. URLs are relative to that directory.
			return imageAgent.getImageFromUrl("webclient_examples/media/dataaccess/textblocks/" + fileName);
		} else {
			// For SiteArchitect, the image file is loaded as a resource with a path relative to the class TextBlocksDataAccessPlugin.
			return imageAgent.getImageFromIcon(new ImageIcon(TextBlocksDataAccessPlugin.class.getResource("resource/icon/" + fileName)));
		}

	}


	/**
	 * Provides a thumbnail for the text block.
	 *
	 * @param object An object representing the given data object.
	 * @param language The language to provide the thumbnail image resource in. If not given, a context related language will be used.
	 * @return A client-specific {@link Image} that contains data about the thumbnail image.
	 */
	@Nullable
	@Override
	public Image<?> getThumbnail(@NotNull final TextBlocksObject object, @Nullable final Language language) {
		final String work = object.getWork();
		final ImageAgent imageAgent = _context.requireSpecialist(ImageAgent.TYPE);

		// Pre-compose the thumbnail file name based on the given work.
		final String fileNameComposite = work + ".jpg";

		// Create an Image object based on the FirstSpirit client's requirements.
		if (_context.is(BaseContext.Env.WEBEDIT) || _context.is(BaseContext.Env.PREVIEW) || _context.is(BaseContext.Env.HEADLESS)) {
			// For use in ContentCreator, image files need to be rolled out to the project-specific ContentCreator web application's
			// directory (this can be initiated by the build.xml). URLs are relative to that directory.
			return imageAgent.getImageFromUrl("webclient_examples/media/dataaccess/textblocks/" + fileNameComposite);
		} else {
			// For SiteArchitect, the image file is loaded as a resource with a path relative to the class ShakespeareDataAccessPlugin.
			final URL url = TextBlocksDataAccessPlugin.class.getResource("resource/icon/" + fileNameComposite);
			if (url == null){
				return null;
			} else {
				return imageAgent.getImageFromIcon(new ImageIcon(url));
			}
		}
	}


	/**
	 * Provides the header text for the given {@link TextBlocksObject}.
	 *
	 * In this particular example implementation, texts are always in English, so the given {@link Language} object is ignored.
	 *
	 * @param object An object representing the given data object.
	 * @param language The language to provide the header text in. If not given, a context related language will be used.
	 * @return A header text.
	 */
	@NotNull
	@Override
	public String getHeader(@NotNull final TextBlocksObject object, @Nullable final Language language) {
		return object.getTitle();
	}


	/**
	 * Provides the extract text for the given {@link TextBlocksObject}.
	 *
	 * In this particular example implementation, texts are always in English, so the given {@link Language} object is ignored.
	 *
	 * @param object An object representing the given data object.
	 * @param language The language to provide the extract text in. If not given, a context related language will be used.
	 * @return An extract text.
	 */
	@NotNull
	@Override
	public String getExtract(@NotNull final TextBlocksObject object, @Nullable final Language language) {
		return object.getText();
	}
}
