package de.espirit.firstspirit.opt.example.universal.dataaccess;

import de.espirit.common.tools.Strings;
import de.espirit.firstspirit.client.plugin.dataaccess.DataStream;
import de.espirit.firstspirit.client.plugin.report.ParameterMap;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksFilterableDataStreamBuilderAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.provider.TextBlocksObjectProvider;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Optional;


/**
 * This class provides a stream of data objects ({@link TextBlocksObject}s) and optionally filters these objects
 * for a limited result set.
 */
public class TextBlocksDataStream implements DataStream<TextBlocksObject> {

	@Nullable
	private Iterator<TextBlocksObject> _textBlocksObjectIterator;
	private final int _total;

	/**
	 * Performs initial retrieval and filtering of text block objects based on filter settings passed as the first parameter.
	 *
	 * @param parameterMap A map of filter settings to consider while retrieving text blocks objects.
	 * @param textBlocksObjectProvider A provider of text block objects (stand-in for a web service or other sort of external data provider).
	 */
	TextBlocksDataStream(@NotNull final ParameterMap parameterMap, @NotNull final TextBlocksObjectProvider textBlocksObjectProvider) {
		// Obtain the full list of text blocks from the TextBlocksObjectProvider object.
		final List<TextBlocksObject> unfilteredTextBlocks = textBlocksObjectProvider.getAllTextBlocks();

		final List<TextBlocksObject> filteredTextBlocks = new ArrayList<>();

		// Retrieve the current filter settings (referencing the constants defined in TextBlocksFilterableDataStreamBuilderAspect)
		// so that we can create a filtered results list, parts of which may be returned by #getNext(int).
		final String searchWords = parameterMap.get(TextBlocksFilterableDataStreamBuilderAspect.SEARCH_WORDS);
		final boolean searchEverywhere = null2false(parameterMap.get(TextBlocksFilterableDataStreamBuilderAspect.IGNORE_FILTER));
		final boolean searchTextOnly = null2false(parameterMap.get(TextBlocksFilterableDataStreamBuilderAspect.SEARCH_IN_TEXT));
		final String selectedGenre = Strings.notNull(parameterMap.get(TextBlocksFilterableDataStreamBuilderAspect.GENRE));
		final String selectedWork = Strings.notNull(parameterMap.get(TextBlocksFilterableDataStreamBuilderAspect.WORKS));

		// Walk through the list "unfilteredTextBlocks" and check if any of the quotations in that list match the filters...
		// Add those quotations that match the filter settings to "filteredTextBlocks".
		for (final TextBlocksObject quotation : unfilteredTextBlocks) {
			if (searchEverywhere || selectedGenre.isEmpty() || workMatchesGenreAndWorkSelection(textBlocksObjectProvider, quotation.getWork(), selectedGenre, selectedWork) ) {
				// Filtering for text occurrences only occurs if "searchWords" contains a text of at least 3 characters in length.
				if (searchWords != null && searchWords.length() > 2) {
					if (searchTextOnly) {
						if (quotation.getText().toUpperCase(Locale.US).contains(searchWords.toUpperCase(Locale.US))) {
							filteredTextBlocks.add(quotation);
						}
					} else {
						if (quotation.getTitle().toUpperCase(Locale.UK).contains(searchWords.toUpperCase(Locale.UK)) ||
								quotation.getText().toUpperCase(Locale.US).contains(searchWords.toUpperCase(Locale.US))) {
							filteredTextBlocks.add(quotation);
						}
					}
				} else {
					// Search words are not defined, so add any quotation.
					filteredTextBlocks.add(quotation);
				}
			}
		}

		// Store the iterator for sortedFilteredTextBlocks in a field so it may be accessed by other methods as they are called later.
		_textBlocksObjectIterator = filteredTextBlocks.iterator();

		// Store the total number of quotations that matched the filter above in a field so it may be accessed later.
		_total = filteredTextBlocks.size();
	}


	private boolean workMatchesGenreAndWorkSelection(@NotNull final TextBlocksObjectProvider textBlocksObjectProvider, final String quotationWork, final String selectedGenre, final String selectedWork) {
		final Optional<List<String>> worksForGenre = textBlocksObjectProvider.getWorks(selectedGenre);
		return worksForGenre.isPresent() && worksForGenre.get().contains(quotationWork) && (selectedWork.isEmpty() || selectedWork.equals(quotationWork));
	}

	/**
	 * Provides up to {@code count} number of quotations that haven't been returned yet.
	 *
	 * @param count The requested number of entries.
	 * @return A {@link List} of {@link TextBlocksObject}s.
	 */
	@NotNull
	@Override
	public List<TextBlocksObject> getNext(final int count) {
		final List<TextBlocksObject> textBlocksObjects = new ArrayList<TextBlocksObject>();
		if (_textBlocksObjectIterator == null) {
			return textBlocksObjects;
		}
		// Add text blocks to textBlocksObjects as long as the count isn't met and the sorted, filtered text blocks list has at least one more entry.
		while (textBlocksObjects.size() < count && _textBlocksObjectIterator.hasNext()) {
			textBlocksObjects.add(_textBlocksObjectIterator.next());
		}
		return textBlocksObjects;
	}


	/**
	 * Indicates if any more text blocks are available that haven't been returned yet.
	 *
	 * @return {@code true} if more text blocks are available that haven't been returned yet, otherwise {@code false}.
	 */
	@Override
	public boolean hasNext() {
		return _textBlocksObjectIterator != null && _textBlocksObjectIterator.hasNext();
	}


	/**
	 * Provides the total number of text blocks which may be returned, based on filter settings.
	 *
	 * @return The total number of available text blocks which match the current filter settings.
	 */
	@Override
	public int getTotal() {
		return _total;
	}


	/**
	 * Performs clean-up.
	 */
	@Override
	public void close() {
		_textBlocksObjectIterator = null;
	}

	// IndexSwingGadget can return Null for unselected ParameterBoolean - see TS#212361
	private static boolean null2false(@Nullable final Boolean variable){
		if (variable != null && variable){
			return true;
		} else {
			return false;
		}
	}
}
