package de.espirit.firstspirit.opt.example.universal.dataaccess;

import de.espirit.firstspirit.client.plugin.dataaccess.DataStream;
import de.espirit.firstspirit.client.plugin.dataaccess.DataStreamBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Filterable;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.StreamBuilderAspectMap;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.StreamBuilderAspectType;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Updating;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksFilterableDataStreamBuilderAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.TextBlocksUpdatingDataStreamBuilderAspect;
import de.espirit.firstspirit.opt.example.universal.dataaccess.provider.TextBlocksObjectProvider;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * This class builds a {@link DataStream DataStream<D>} object and specifies aspects which advertise various features
 * related to providing a stream of data objects.
 */
public class TextBlocksDataStreamBuilder implements DataStreamBuilder<TextBlocksObject> {

	@NotNull
	private final TextBlocksObjectProvider _textBlocksObjectProvider;
	private final StreamBuilderAspectMap _aspects;
	private final TextBlocksFilterableDataStreamBuilderAspect _filterable;
	private final TextBlocksUpdatingDataStreamBuilderAspect _updating;


	/**
	 * Instantiates a {@link StreamBuilderAspectMap} and registers appropriate aspect objects in that map.
	 *
	 * @param textBlocksObjectProvider The object which serves as an external source for text blocks data.
	 */
	public TextBlocksDataStreamBuilder(@NotNull final TextBlocksObjectProvider textBlocksObjectProvider) {
		_textBlocksObjectProvider = textBlocksObjectProvider;
		_aspects = new StreamBuilderAspectMap();

		// Add data stream builder-specific aspects to the aspect map.

		// Advertise that the data stream supports filters. This causes a filter UI to be displayed with
		// UI representations of this DataAccessPlugin and allows the data stream to filter text blocks by text search.
		_filterable = new TextBlocksFilterableDataStreamBuilderAspect();
		_updating = new TextBlocksUpdatingDataStreamBuilderAspect();
		_aspects.put(Updating.TYPE, _updating);

		_aspects.put(Filterable.TYPE, _filterable);

	}


	/**
	 * Provides the requested aspect object, if available.
	 *
	 * @param type The type of aspect.
	 * @param <A> The requested aspect interface.
	 * @return An implementation of {@code <A>}, or {@code null} if no aspect of that type exists in the aspect map.
	 */
	@Nullable
	@Override
	public <A> A getAspect(@NotNull final StreamBuilderAspectType<A> type) {
		return _aspects.get(type);
	}


	/**
	 * Instantiates and provides a {@link TextBlocksDataStream} object which will serve as a provider of {@link TextBlocksObject}s.
	 *
	 * @return A {@link DataStream DataStream<D>}.
	 */
	@NotNull
	@Override
	public DataStream<TextBlocksObject> createDataStream() {
		// For simplicity, the TextBlocksDataStream constructor receives a ParameterMap and a TextBlocksObjectProvider so
		// it doesn't need to source these from somewhere else.
		return new TextBlocksDataStream(_filterable.getFilter(), _textBlocksObjectProvider);
	}

}
