package de.espirit.firstspirit.opt.example.universal.dataaccess;

import org.jetbrains.annotations.NotNull;


/**
 * This class is the data object that is passed between classes related to {@link TextBlocksDataAccessPlugin}. It stores
 * a numeric ID, a title string and a text string.
 */
public class TextBlocksObject {

	private final String _id;
	private final String _title;
	private final String _text;

	/**
	 * Configures a {@link TextBlocksObject} with the given ID, title, and text.
	 *
	 * @param id The ID.
	 * @param title The text block's title.
	 * @param text The text block's text.
	 */
	public TextBlocksObject(final String id, @NotNull final String title, @NotNull final String text) {
		_id = id;
		_title = title;
		_text = text;
	}

	/**
	 * Provides the text block object's ID.
	 *
	 * @return The ID.
	 */
	public String getId() {
		return _id;
	}


	/**
	 * Provides the text block object's title.
	 *
	 * @return The title.
	 */
	@NotNull
	public String getTitle() {
		return _title;
	}


	/**
	 * Provides the work (book) this quote is from.
	 *
	 * @return The work.
	 */
	public String getWork() {
		return _title.substring(0, _title.indexOf(',')); // Get work's title from TextBlock title
	}


	/**
	 * Provides the text block object's text.
	 *
	 * @return The text.
	 */
	@NotNull
	public String getText() {
		return _text;
	}


	/**
	 * Generates a hash code based on the text block object's internal information.
	 *
	 * @return The hash code.
	 */
	public int hashCode() {
		return 31 ^ getId().hashCode() ^ getTitle().hashCode() ^ getText().hashCode();
	}


	/**
	 * Determines equality with another {@link Object}.
	 *
	 * Equality is only given if the provided {@code object} is of type {@link TextBlocksObject} and IDs, title string, and text string match with those of this instance.
	 *
	 * @param object The object to determine equality with.
	 * @return {@code true} if the given object equals this instance, otherwise {@code false}.
	 */
	public boolean equals(final Object object) {
		// Equality between this instance and the object passed as a parameter is given if the object is also of type TextBlocksObject
		// and IDs, title strings, and text strings are equal.
		return object instanceof TextBlocksObject &&
				getId().equals(((TextBlocksObject) object).getId()) &&
				getTitle().equals(((TextBlocksObject) object).getTitle()) &&
				getText().equals(((TextBlocksObject) object).getText());
	}
}
