package de.espirit.firstspirit.opt.example.universal.dataaccess.aspects;

import de.espirit.firstspirit.client.plugin.dataaccess.DataStreamBuilder;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Filterable;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ParameterBoolean;
import de.espirit.firstspirit.client.plugin.report.ParameterMap;
import de.espirit.firstspirit.client.plugin.report.ParameterSelect;
import de.espirit.firstspirit.client.plugin.report.ParameterText;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksDataStream;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksDataStreamBuilder;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * {@link DataStreamBuilder DataStreamBuilder<D>} Aspect: {@link Filterable}
 *
 * Advertises that the {@link TextBlocksDataStream} class instantiated by {@link TextBlocksDataStreamBuilder} supports filtering capabilities
 * and specifies parameters which will be made available in the data access plug-in's filter UI.
 */
public class TextBlocksFilterableDataStreamBuilderAspect implements Filterable {
	// Definition of search parameter constants which specify parameter identifiers centrally.
	public static final ParameterText SEARCH_WORDS = Parameter.Factory.createText("searchWords", null, null);
	public static final ParameterBoolean IGNORE_FILTER = Parameter.Factory.createBoolean("ignoreFilter", null, false);
	public static final ParameterBoolean SEARCH_IN_TEXT = Parameter.Factory.createBoolean("searchInText", null, false);
	public static final ParameterSelect GENRE = Parameter.Factory.createSelect("genre", null, null);
	public static final ParameterSelect WORKS = Parameter.Factory.createSelect("works", null, null);

	private ParameterMap _filter;


	/**
	 * Creates {@link Parameter Parameter<T>} objects which will be used in the filtering UI of Text Blocks reports and
	 * text block selection dialogs used in FS_INDEX input components.
	 *
	 * @return A {@link List} of {@link Parameter Parameter<T>} objects.
	 */
	@NotNull
	@Override
	public List<Parameter<?>> getDefinedParameters() {
		final List<Parameter<?>> parameters = new ArrayList<Parameter<?>>();

		// Configures a text parameter in which search words may be entered. Default value is "".
		final ParameterText searchWords = Parameter.Factory.createText(SEARCH_WORDS.getName(), "(Search for...)", "");
		parameters.add(searchWords);

		// A selection of genres (by default 'Comedy' and 'Tragedy')
		final ParameterSelect.SelectItem genre0 = Parameter.Factory.createSelectItem("[genre]", "");
		final ParameterSelect.SelectItem genre1 = Parameter.Factory.createSelectItem("Comedy", "Comedy");
		final ParameterSelect.SelectItem genre2 = Parameter.Factory.createSelectItem("Tragedy", "Tragedy");
		final ParameterSelect genre = Parameter.Factory.createSelect(GENRE.getName(), Arrays.asList(genre0, genre1, genre2), "");
		parameters.add(genre);

		// A selection of works from the selected genre - will be filled/ updated upon genre selection
		final ParameterSelect.SelectItem works0 = Parameter.Factory.createSelectItem("[work]", "");
		final ParameterSelect works = Parameter.Factory.createSelect(WORKS.getName(), Arrays.asList(works0), "");
		parameters.add(works);

		// A switch enabling a 'filter bypass', i.e. lets the plugin search everywhere despite filter selections
		final ParameterBoolean searchEverywhere = Parameter.Factory.createBoolean(IGNORE_FILTER.getName(), "Ignore search filter selection", false);
		parameters.add(searchEverywhere);

		// Configures a boolean parameter which controls if search words should only be searched in the text block's text field. Default value is "false".
		final ParameterBoolean searchInText = Parameter.Factory.createBoolean(SEARCH_IN_TEXT.getName(), "Only search in text", false);
		parameters.add(searchInText);

		return parameters;
	}


	/**
	 * Stores the given {@link ParameterMap} internally so it may be retrieved later.
	 *
	 * @param filter The filter settings.
	 */
	@Override
	public void setFilter(@NotNull final ParameterMap filter) {
		_filter = filter;
	}


	/**
	 * Provides the internally stored {@link ParameterMap} object.
	 *
	 * @return A {@link ParameterMap} representing the current filter settings.
	 */
	public ParameterMap getFilter() {
		return _filter;
	}

}
