package de.espirit.firstspirit.opt.example.universal.dataaccess.aspects;

import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.Identifying;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;
import de.espirit.firstspirit.opt.example.universal.dataaccess.provider.TextBlocksObjectProvider;

import org.jetbrains.annotations.NotNull;

import java.util.NoSuchElementException;
import java.util.Optional;


/**
 * {@link DataAccessSession DataAccessSession<D>} Aspect: {@link Identifying}
 *
 * Advertises that this data access plug-in can match objects with identifying features of the data objects it works with.
 *
 * In this example, we can compare {@link TextBlocksObject}s for equality directly, but we may also check if a given integer is the ID of a
 * text blocks object, or if a given string happens to be the exact title of a text block (which, as generated by our data source, the
 * {@link TextBlocksObjectProvider}, is actually unique as well, because it uses a fixed text and the text block's ID to build the title).
 */
public class TextBlocksIdentifyingDataAccessSessionAspect implements Identifying {

	private final TextBlocksObjectProvider _textBlocksObjectProvider;


	public TextBlocksIdentifyingDataAccessSessionAspect(@NotNull final TextBlocksObjectProvider textBlocksObjectProvider) {
		_textBlocksObjectProvider = textBlocksObjectProvider;
	}


	/**
	 * Attempts to find a matching {@link TextBlocksObject} as provided by {@link TextBlocksObjectProvider} using the given {@code object}.
	 *
	 * @param object Some object.
	 * @return A {@link String} identifier for a valid {@link TextBlocksObject}.
	 * @throws NoSuchElementException if no valid {@link TextBlocksObject} could be identified from the given data.
	 */
	@Override
	public String getIdentifier(final Object object) throws NoSuchElementException {

		// We can infer identifying data from the object depending on the object's type.
		if (object instanceof String) {
			final Optional<TextBlocksObject> textBlocksObjectOptional = _textBlocksObjectProvider.getSingleTextBlock((String)object);
			if (textBlocksObjectOptional.isPresent()){ // maybe a TextBlocks ID was provided?
				return textBlocksObjectOptional.get().getId();

			}
		} else if (object instanceof TextBlocksObject) {
			// The given object is a TextBlocksObject. Try to retrieve its equal from the TextBlocksObjectProvider.
			final Optional<TextBlocksObject> retrievedTextBlocksObject = _textBlocksObjectProvider.getSingleTextBlock(((TextBlocksObject) object).getId());
			if (retrievedTextBlocksObject.isPresent()){
				return retrievedTextBlocksObject.get().getId();
			}
		}

		// The given object did not identify a TextBlocksObject. Throw an exception.
		throw new NoSuchElementException("Provided object did not identify any valid " + TextBlocksObject.class.toString() + '.');
	}
}
