package de.espirit.firstspirit.opt.example.universal.dataaccess.aspects;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.agency.TransferAgent;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.transfer.SupplierHost;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.transfer.TransferSupplying;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;
import de.espirit.firstspirit.ui.gadgets.aspects.transfer.TransferType;

import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;


/**
 * {@link DataAccessSession DataAccessSession<D>} Aspect: {@link TransferSupplying TransferSupplying<D>}

 * This class advertises that the data access plug-in can provide data about text block entries which may be used in drag-and-drop operations
 * originating from a Text Block report or from an FS_INDEX input component using this data access plug-in as a data source, for example.
 * It registers supplier objects which provide data of different flavors.
 */
public class TextBlocksTransferSupplyingDataAccessSessionAspect implements TransferSupplying<TextBlocksObject> {

	private final BaseContext _context;


	public TextBlocksTransferSupplyingDataAccessSessionAspect(@NotNull final BaseContext context) {
		_context = context;
	}


	/**
	 * Instantiates and registers supplier objects which can provide data of various flavors for use in drag-and-drop operations.
	 *
	 * @param host The host to register at.
	 */
	@Override
	public void registerSuppliers(@NotNull final SupplierHost<TextBlocksObject> host) {
		final TransferAgent transferAgent = _context.requireSpecialist(TransferAgent.TYPE);

		// Instantiate and register a supplier object that supplies data of type TextBlocksObject.
		// This enables Text Blocks reports and FS_INDEX input components which use this data access plug-in as a data source
		// to serve as drag-and-drop origins from which text block entries may be dragged onto other FS_INDEX components using this plug-in.
		final TransferType<TextBlocksObject> textBlocksObjectTransferType = transferAgent.getRawValueType(TextBlocksObject.class);
		host.registerSupplier(textBlocksObjectTransferType, new TextBlocksObjectSupplier());

		// Instantiate and register a supplier object that supplies data of type String.
		final TransferType<String> stringObjectTransferType = transferAgent.getStringType();
		host.registerSupplier(stringObjectTransferType, new StringObjectSupplier());

		// Instantiate and register a supplier object that supplies data of type String, MIME type "text/html".
		final TransferType<String> htmlObjectTransferType = transferAgent.getHtmlTextType();
		host.registerSupplier(htmlObjectTransferType, new HTMLObjectSupplier());

		// Instantiate and register a supplier object that supplies data of type String, MIME type "text/plain".
		final TransferType<String> plainTextObjectTransferType = transferAgent.getPlainTextType();
		host.registerSupplier(plainTextObjectTransferType, new PlainTextObjectSupplier());
	}


	/**
	 * Supplier class which provides objects of type {@link TextBlocksObject}.
	 */
	private static class TextBlocksObjectSupplier implements SupplierHost.Supplier<TextBlocksObject, TextBlocksObject> {

		@Override
		public List<TextBlocksObject> supply(@NotNull final TextBlocksObject object) {
			return Collections.singletonList(object);
		}

	}


	/**
	 * Supplier class which provides objects of type {@link String}.
	 */
	private static class StringObjectSupplier implements SupplierHost.Supplier<TextBlocksObject, String> {

		@Override
		public List<String> supply(@NotNull final TextBlocksObject object) {
			return Arrays.asList(
					object.getTitle(),
					object.getText()
			);
		}
	}


	/**
	 * Supplier class which provides objects of type {@link String}, optimized for the MIME type "text/html".
	 */
	private static class HTMLObjectSupplier implements SupplierHost.Supplier<TextBlocksObject, String> {

		@Override
		public List<String> supply(@NotNull final TextBlocksObject object) {
			return Collections.singletonList("<h2>" + object.getTitle() + "</h2><p>" + object.getText() + "</p>");
		}
	}


	/**
	 * Supplier class which provides objects of type {@link String}, optimized for the MIME type "text/plain".
	 */
	private static class PlainTextObjectSupplier implements SupplierHost.Supplier<TextBlocksObject, String> {

		@Override
		public List<String> supply(@NotNull final TextBlocksObject object) {
			return Collections.singletonList(object.getText());
		}
	}
}
