package de.espirit.firstspirit.opt.example.universal.dataaccess.aspects;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.ValueIndexer;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.ValueIndexing;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;
import de.espirit.firstspirit.opt.example.universal.dataaccess.provider.TextBlocksObjectProvider;

import org.jetbrains.annotations.NotNull;

import java.util.Optional;


/**
 * {@link DataAccessSession DataAccessSession<D>} Aspect: {@link ValueIndexing}
 *
 * This class provides indexing functionality so that text content of {@link TextBlocksObject}s referenced by
 * FS_INDEX input components and elsewhere can be found using search.
 */
public class TextBlocksValueIndexingDataAccessSessionAspect implements ValueIndexing {

	private final TextBlocksObjectProvider _textBlocksObjectProvider;


	/**
	 * Stores a {@link TextBlocksObjectProvider} internally so that {@link TextBlocksObject}s may be retrieved using the
	 * string identifiers given to {@link #appendIndexData(String, Language, boolean, ValueIndexer)}.
	 *
	 * @param textBlocksObjectProvider The {@link TextBlocksObjectProvider} used by the associated data access session.
	 */
	public TextBlocksValueIndexingDataAccessSessionAspect(@NotNull final TextBlocksObjectProvider textBlocksObjectProvider) {
		_textBlocksObjectProvider = textBlocksObjectProvider;
	}


	/**
	 * Submits data for indexing.
	 *
	 * @param identifier The identifier of the data object to be indexed.
	 * @param language The language to index for.
	 * @param recursive If {@code true}, index referenced sub-objects as well.
	 * @param indexer The indexer to be used for appending data.
	 */
	@Override
	public void appendIndexData(final String identifier, final Language language, final boolean recursive, final ValueIndexer indexer) {
		final Optional<TextBlocksObject> textBlocksObject = _textBlocksObjectProvider.getSingleTextBlock(identifier);
		if (textBlocksObject.isPresent()) {
			// Add both title and text strings of the text blocks object to the index.
			// We're using the constant ValueIndexer.KEY_FIELD, which is for language-independent data.
			indexer.append(ValueIndexer.KEY_FIELD, textBlocksObject.get().getTitle());
			indexer.append(ValueIndexer.KEY_FIELD, textBlocksObject.get().getText());
		}
	}
}
