package de.espirit.firstspirit.opt.example.universal.dataaccess.aspects.reporting;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.agency.Image;
import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessSession;
import de.espirit.firstspirit.client.plugin.dataaccess.aspects.DataTemplating;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksDataSnippetProvider;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * {@link DataAccessSession DataAccessSession<D>} Aspect: {@link DataTemplating DataTemplating<D>}
 *
 * Implements support for details fly-outs in ContentCreator when {@link TextBlocksObject} representations are shown
 * in a report UI and receive MouseOver events.
 */
public class TextBlocksDataTemplatingDataAccessSessionAspect implements DataTemplating<TextBlocksObject> {

	private final TextBlocksDataSnippetProvider _snippetProvider;

	// Identifiers used for parameters. The identifiers will be used in #getTemplate(...) to mark spots in HTML where
	// the parameters registered in #registerParameters(...) should be inserted.
	private static final String IDENTIFIER_TEXTBLOCK_TITLE = "textblock_title";
	private static final String IDENTIFIER_TEXTBLOCK_TEXT = "textblock_text";
	private static final String IDENTIFIER_TEXTBLOCK_THUMBNAIL = "textblock_thumbnail";


	public TextBlocksDataTemplatingDataAccessSessionAspect(@NotNull final BaseContext context) {
		// Instead of ad-hoc rendering of text block data, we'll use the TextBlocksDataSnippetProvider class.
		_snippetProvider = new TextBlocksDataSnippetProvider(context);
	}


	/**
	 * Provides a string which specifies HTML that is to be shown in the details fly-out.
	 *
	 * @param object The object to be represented.
	 * @param language The language to get the template for (NOTE that we are not using languages, since all texts in this plug-in are in English).
	 * @return HTML code.
	 */
	@Override
	public String getTemplate(@NotNull final TextBlocksObject object, @NotNull final Language language) {
		final StringBuilder htmlString = new StringBuilder();
		htmlString.append("<h3>").append("${").append(IDENTIFIER_TEXTBLOCK_TITLE).append('}').append("</h3>");
		htmlString.append("<style>div.gwt-PopupPanel { width:33%;max-width:40rem; }");
		// Only append thumbnail div (and style) if thumbnail exists:
		if (_snippetProvider.getThumbnail(object, language) != null) {
			htmlString.append("\ndiv.gwt-PopupPanel img { max-width: 100%; }</style>");
			htmlString.append("<div style=\"text-align:center;\">${").append(IDENTIFIER_TEXTBLOCK_THUMBNAIL).append("}</div>");
		} else {
			htmlString.append("</style>");
		}
		htmlString.append("${").append(IDENTIFIER_TEXTBLOCK_TEXT).append('}');
		return htmlString.toString();
	}


	/**
	 * Registers parameters for a {@link TextBlocksObject}'s title, text, and thumbnail. The identifiers placed in the HTML provided by {@link #getTemplate(TextBlocksObject, Language)}
	 * will be replaced with the values of the parameters registered here.
	 *
	 * @param parameters The parameters container to register parameters with.
	 * @param object The object to ask for parameter values.
	 * @param language The language to get the values in (NOTE that we are not using languages, since all texts in this plug-in are in English).
	 */
	@Override
	public void registerParameters(@NotNull final ParameterSet parameters, @NotNull final TextBlocksObject object, @NotNull final Language language) {
		parameters.addText(IDENTIFIER_TEXTBLOCK_TITLE, _snippetProvider.getHeader(object, language));
		parameters.addText(IDENTIFIER_TEXTBLOCK_TEXT, _snippetProvider.getExtract(object, language));

		// The DataTemplating aspect should only be used in ContentCreator for now. Just to be safe, though, we only register
		// this parameter if a thumbnail is available and the image's source is specified as a String (i.e. a URL for use in ContentCreator).
		// The ContentCreator (imageAgent.getImageFromUrl) does not check whether an image does or does not exist! Missing images will
		// lead to 404 errors in the webbrowser, and to report fly-outs not being displayed.
		final Image<?> thumbnail = _snippetProvider.getThumbnail(object, language);
		if (thumbnail != null) {
			thumbnail.supply(new Image.Consumer<Void>() {
				@Nullable
				@Override
				public Void consume(@NotNull String url) {
					parameters.addImage(IDENTIFIER_TEXTBLOCK_THUMBNAIL, thumbnail, "");
					return null;
				}
			});
		}
	}
}
