package de.espirit.firstspirit.opt.example.universal.dataaccess.provider;

import de.espirit.common.base.Logging;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksDataAccessPlugin;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;

import javax.validation.constraints.NotNull;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class reads quotations from provided Shakespeare works. The categorization in genres is determined by file paths.
 */
public class ShakespeareReader {

	private final Map<String,TextBlocksObject> _quotations;
	private final Map<String, ArrayList<String>> _genres;

	public ShakespeareReader(){
		_quotations = new TreeMap<>();
		_genres = new TreeMap<>();
		// Resource paths have to end in [genre]/[work]/[filename] for readWork(URL) to work properly!
		readWork("resource/works/Comedy/As You Like It/full.html");
		readWork("resource/works/Comedy/All's Well/full.html");
		readWork("resource/works/Tragedy/King Lear/full.html");
		readWork("resource/works/Tragedy/Romeo and Juliet/full.html");
	}

	public Map<String,TextBlocksObject> getQuotations(){
		return _quotations;
	}

	public Map<String, ArrayList<String>> getGenres(){
		return _genres;
	}

	// This method attempts to read individual quotations from the provided resource.
	// Genre and work title for all quotation objects are extracted from the resource URL.
	// Genres and works belonging to each genre are collected in _genres.
	private void readWork(@NotNull final String path) {
		final URL resource = TextBlocksDataAccessPlugin.class.getResource(path);
		if (resource == null){
			Logging.logError("Requested text resource '" + path + "' could not be found!", TextBlocksObjectProvider.class);
			return;
		}
		final ParseShakespeare parser = new ParseShakespeare();
		final String[] parts_of_string = resource.getPath().split("/");
		final String genre = parts_of_string[parts_of_string.length - 3];
		final String work = parts_of_string[parts_of_string.length - 2].replaceAll("%20", " ");
		if (!_genres.containsKey(genre)) {
			_genres.put(genre, new ArrayList<>());
		}
		_genres.get(genre).add(work); // add to genre's list of works
		try (final BufferedReader br = new BufferedReader(new InputStreamReader(resource.openStream()))) {
			String line;
			while ((line = br.readLine()) != null) {
				if (parser.process(line, work)){
					final TextBlocksObject textBlocksObject = parser.collect();
					_quotations.put(textBlocksObject.getId(), textBlocksObject);
				}
			}
		} catch (final IOException fn){
			Logging.logError("Problem reading " + resource, TextBlocksObjectProvider.class);
		}
	}


	private class ParseShakespeare{
		private boolean _collectQuote;
		private String _speaker = "";
		private StringBuffer _quote;
		private int _act;
		private int _scene;
		private int _line;
		private final Pattern pattern = Pattern.compile("<A NAME=(\\d+)\\.(\\d+)\\.(\\d+)>(.*?)</A><br>");
		private TextBlocksObject _textBlocksObject;

		ParseShakespeare(){
			_quote = new StringBuffer();
		}

		public boolean process(final String line, final String work){
			boolean collect = false;
			if ("<blockquote>".equals(line)){
				_collectQuote = true;
			} else if ("</blockquote>".equals(line)){
				_collectQuote = false;
				if (_act > 0) {
					final String title = work + ", Act " + _act + ", Scene " + _scene + ", " + _speaker;
					final String key = work.substring(work.lastIndexOf(' ') + 1) + _act + 's' + _scene + 'l' + _line;
					_textBlocksObject = new TextBlocksObject(key, title, _quote.toString());
					collect = true;
				}
				_quote = new StringBuffer();
			} else if (line.startsWith("<A NAME=speech")){
				_speaker = line.replaceAll("<[^>]*>",""); // strip tags
			} else if (_collectQuote){
				if (_quote.length() > 0){
					_quote.append('\n');
				}
				final Matcher matcher = pattern.matcher(line);
				if (matcher.find()) {
					_act = Integer.parseInt(matcher.group(1));
					_scene = Integer.parseInt(matcher.group(2));
					_line = Integer.parseInt(matcher.group(3));
					_quote.append(matcher.group(4));
				}
			}
			return collect;
		}


		public TextBlocksObject collect(){
			return _textBlocksObject;
		}

	}

}
