package de.espirit.firstspirit.opt.example.universal.dataaccess.provider;

import de.espirit.firstspirit.client.plugin.dataaccess.DataAccessPlugin;
import de.espirit.firstspirit.opt.example.universal.dataaccess.TextBlocksObject;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.TreeMap;


/**
 * Provider for {@link TextBlocksObject}s. This class is used in lieu of a web service or other external means of providing
 * data to display and use via a {@link DataAccessPlugin} implementation.
 *
 * The provider can provide a list of all text block objects (stored in a private field) and may also be asked to provide a
 * single text block object based on the numeric ID.
 */
public class TextBlocksObjectProvider {

	private final Map<String,TextBlocksObject> _textBlocksObjects;

	private final Map<String, ArrayList<String>> _genres;

	/**
	 * Creates a new {@link TreeMap} and populates it with {@link TextBlocksObject}s using {@link #populateList()}.
	 */
	public TextBlocksObjectProvider() {
		final ShakespeareReader shakespeareReader = new ShakespeareReader();
		_textBlocksObjects = shakespeareReader.getQuotations();
		_genres = shakespeareReader.getGenres();
	}


	/**
	 * Getter method for a single {@link TextBlocksObject}.
	 *
	 * @param id The ID of the desired text block object.
	 * @return An Optional containing a {@link TextBlocksObject} matching the given ID, or empty if no text block with that ID exists.
	 */
	public Optional<TextBlocksObject> getSingleTextBlock(@NotNull final String id) {
		return Optional.ofNullable(_textBlocksObjects.get(id));
	}


	/**
	 * Getter method for the entire list of available {@link TextBlocksObject}.
	 *
	 * @return A {@link List} of {@link TextBlocksObject}s.
	 */
	public List<TextBlocksObject> getAllTextBlocks() {
		return new ArrayList<>(_textBlocksObjects.values());
	}


	/**
	 * Getter method for a set of all genres.
	 *
	 * @return A {@link Set} of {@link String}s.
	 */
	@NotNull
	Set<String> getAllGenres(){
		return _genres.keySet();
	}


	/**
	 * Getter method for the list of all works from a given genre.
	 *
	 * @param genre The name of a genre.
	 * @return A {@link List} of {@link String}s.
	 */
	public Optional<List<String>> getWorks(@NotNull final String genre){
		return Optional.ofNullable(_genres.get(genre));
	}


	/**
	 * Getter method for a list of Shakespeare quotations from a specific work.
	 *
	 * @param work The name of a work (a book).
	 * @return A {@link List} of {@link TextBlocksObject}s.
	 */
	@NotNull
	List<TextBlocksObject> getWork(@NotNull final String work){
		final ArrayList<TextBlocksObject> resultList = new ArrayList<>();
		_textBlocksObjects.forEach((k, v) -> { if (v.getWork().equals(work)){ resultList.add(v); } } );
		return resultList;
	}

}