package de.espirit.firstspirit.opt.example.universal.gadget;

import de.espirit.common.Default;
import de.espirit.common.GomDoc;
import de.espirit.common.InheritAnnotations;
import de.espirit.common.Mandatory;
import de.espirit.firstspirit.access.store.templatestore.gom.AbstractGomElement;
import de.espirit.firstspirit.access.store.templatestore.gom.AbstractGomFormElement;
import de.espirit.firstspirit.access.store.templatestore.gom.DefaultGomLangInfos;
import de.espirit.firstspirit.access.store.templatestore.gom.GomElement;
import de.espirit.firstspirit.access.store.templatestore.gom.GomIncludeConfiguration;
import de.espirit.firstspirit.access.store.templatestore.gom.GomLangInfo;
import de.espirit.firstspirit.access.store.templatestore.gom.GomLangInfoProvider;
import de.espirit.firstspirit.access.store.templatestore.gom.GomList;
import de.espirit.firstspirit.access.store.templatestore.gom.YesNo;

import org.jetbrains.annotations.Nullable;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


/**
 * GOM syntax definition for an input component "EXAMPLE_PHONENUMBER".
 *
 * The GOM XML supports the following structure:
 *
 * <EXAMPLE_PHONENUMBER
 *     name="[string]"
 *     defaultCountryCode="[number]"
 *     requireCountryCode="[YES|NO]">
 *     <COUNTRIES>
 *         <COUNTRY code="[number]">
 *             <LANGINFOS>
 *                 <LANGINFO lang="[string]" label="[string]" />
 *                 ...
 *             </LANGINFOS>
 *         </COUNTRY>
 *         ...
 *     </COUNTRIES>
 *     <LANGINFOS>
 *         <LANGINFO lang="[string]" label="[string]" />
 *         ...
 *     </LANGINFOS>
 * </EXAMPLE_PHONENUMBER>
 */
public class GomPhoneNumber extends AbstractGomFormElement {

	private static final long serialVersionUID = -8825642613892722377L;

	// TAG must equal the name of the component in module.xml which defines this plug-in.
	public static final String TAG = "EXAMPLE_PHONENUMBER";

	// Comparator on country entries
	private static final CountryComparator COUNTRY_COMPARATOR = new CountryComparator();

	private YesNo _requireCountryCode;
	private Integer _defaultCountryCode;
	private Countries _codes;


	@Override
	protected String getDefaultTag() {
		return TAG;
	}


	/**
	 * Getter method for the <EXAMPLE_PHONENUMBER /> attribute "hFill".
	 *
	 * This getter method returns a hardcoded value so that the component is always shown in forms at maximum width.
	 *
	 * Note: the setter method is implemented by the class {@link AbstractGomFormElement}.
	 *
	 * @return {@link YesNo#YES}
	 */
	@Override
	@InheritAnnotations
	public YesNo getHFill() {
		return YesNo.YES;
	}


	/**
	 * Getter method for the <EXAMPLE_PHONENUMBER /> attribute "noBreak".
	 *
	 * This getter method returns a hardcoded value so that the component is always drawn with a following line break in forms.
	 *
	 * Note: the setter method is implemented by the class {@link AbstractGomFormElement}.
	 *
	 * @return {@link YesNo#NO}
	 */
	@Override
	@InheritAnnotations
	public YesNo getNoBreak() {
		return YesNo.NO;
	}


	/**
	 * Getter method for the <EXAMPLE_PHONENUMBER /> attribute "requireCountryCode".
	 *
	 * @return Is selection of a country code required? ({@link YesNo#YES}/{@link YesNo#NO})
	 */
	@GomDoc(description="Require country code.", since="1.0.0")
	@Default("no")
	public YesNo getRequireCountryCode() {
		return _requireCountryCode;
	}


	/**
	 * Setter method for the <EXAMPLE_PHONENUMBER /> attribute "requireCountryCode".
	 *
	 * @param requireCountryCode Is selection of a country code required? ({@link YesNo#YES}/{@link YesNo#NO})
	 */
	@SuppressWarnings("UnusedDeclaration")
	public void setRequireCountryCode(@Nullable final YesNo requireCountryCode) {
		_requireCountryCode = requireCountryCode;
	}


	/**
	 * Getter method for the <EXAMPLE_PHONENUMBER /> attribute "defaultCountryCode".
	 *
	 * @return The default country code. ({@link Integer})
	 */
	@GomDoc(description="Set the default country code (used for initial combobox display).", since="1.0.0")
	public Integer getDefaultCountryCode() {
		return _defaultCountryCode;
	}


	/**
	 * Setter method for the <EXAMPLE_PHONENUMBER /> attribute "defaultCountryCode".
	 *
	 * @param defaultCountryCode The default country code. ({@link Integer})
	 */
	@SuppressWarnings("UnusedDeclaration")
	public void setDefaultCountryCode(@Nullable final Integer defaultCountryCode) {
		_defaultCountryCode = defaultCountryCode;
	}


	/**
	 * Getter method for the <COUNTRIES /> XML tag, which is a direct descendant of <EXAMPLE_PHONENUMBER />.
	 *
	 * @return A {@link Countries} object.
	 */
	@GomDoc(description = "Add countries whose international codes are supported.", since = "1.0.0")
	public Countries getCountries() {
		if (_codes == null) {
			_codes = new Countries();
		}
		if (!_codes.isEmpty()) {
			Collections.sort(_codes, COUNTRY_COMPARATOR);
		}
		return _codes;
	}


	/**
	 * Definition of the class {@link Countries} used by the getter method {@link #getCountries()}.
	 *
	 * This class extends {@link ArrayList} and implements {@link GomList}.
	 */
	public static class Countries extends ArrayList<Country> implements GomList<Country> {

		private static final long serialVersionUID = 3221254738632146326L;

		public static final String DEFAULT_TAG = "COUNTRIES";
		private String _tag;

		private GomIncludeConfiguration _configuration;
		private Map<String, Class<? extends GomElement>> _mappings;


		public Countries() {
			setGomElementTag(DEFAULT_TAG);
		}


		/**
		 * Provide a map object which associates XML tags which may be used within a <COUNTRIES /> block with classes
		 * implementing these tags.
		 *
		 * @return A {@link Map} associating {@link String} tags with a {@link GomElement} implementation each.
		 */
		@Override
		public Map<String, Class<? extends GomElement>> getGomElementMappings() {
			if (_mappings == null) {
				_mappings = new HashMap<String, Class<? extends GomElement>>();
				_mappings.put(Country.TAG, Country.class);
			}
			return _mappings;
		}


		@Override
		public void setGomElementTag(final String name) {
			_tag = name;
		}


		@Override
		public String getGomElementTag() {
			return _tag;
		}


		@Override
		public GomIncludeConfiguration getIncludeConfiguration() {
			return _configuration;
		}


		@Override
		public void setIncludeConfiguration(final GomIncludeConfiguration configuration) {
			_configuration = configuration;
		}


		@Override
		public boolean hasRepresentation() {
			return false;
		}


		/**
		 * Generate a string in JavaScript Object Notation (JSON) which may be used to transfer the list of country codes
		 * (<COUNTRY /> tags) specified in GOM XML in serialized form (e.g. to the JavaScript controller object used by
		 * the ContentCreator gadget).
		 *
		 * @return A {@link String} in JSON format.
		 */
		public String toJSON() {
			final StringBuilder jsonList = new StringBuilder();
			jsonList.append('[');
			final Iterator<Country> countryIterator = iterator();
			while (countryIterator.hasNext()) {
				final Country country = countryIterator.next();
				jsonList.append('{');

				// Country code
				jsonList.append('"').append("code").append('"').append(':').append('"').append(country.getCode()).append('"').append(',');

				// Language information
				jsonList.append('"').append("langinfos").append('"').append(':');
				jsonList.append('[');
				final Iterator<GomLangInfo> langInfoIterator = country.getLangInfos().iterator();
				while (langInfoIterator.hasNext()) {
					final GomLangInfo langInfo = langInfoIterator.next();
					jsonList.append('{');

					// Language abbreviation
					jsonList.append('"').append("lang").append('"').append(':').append('"').append(langInfo.getLang()).append('"').append(',');

					// Label
					jsonList.append('"').append("label").append('"').append(':').append('"').append(langInfo.getLabel()).append('"').append(',');

					// Description
					jsonList.append('"').append("description").append('"').append(':').append('"').append(langInfo.getDescription()).append('"');

					jsonList.append('}');
					if (langInfoIterator.hasNext()) {
						jsonList.append(',');
					}
				}

				jsonList.append(']');

				jsonList.append('}');
				if (countryIterator.hasNext()) {
					jsonList.append(',');
				}
			}
			jsonList.append(']');
			return jsonList.toString();
		}
	}


	/**
	 * Definition of the <COUNTRY /> XML tag, which is used as a direct descendant of <COUNTRIES />.
	 */
	@GomDoc(description = "A country code definition", since = "1.0.0")
	public static final class Country extends AbstractGomElement implements GomLangInfoProvider {

		private static final long serialVersionUID = -2385826018865464263L;

		public static final String TAG = "COUNTRY";

		private Integer _code;
		private CountryLangInfos _langinfos;


		@Override
		public String getDefaultTag() {
			return TAG;
		}


		@Override
		public boolean hasRepresentation() {
			return false;
		}


		/**
		 * Getter method for the <COUNTRY /> attribute "code".
		 *
		 * @return The country code. ({@link Integer})
		 */
		@GomDoc(description = "The international dialing code", since = "1.0.0")
		@Mandatory
		public Integer getCode() {
			return _code;
		}


		/**
		 * Setter method for the <COUNTRY /> attribute "code".
		 *
		 * @param code The country code. ({@link Integer})
		 */
		public void setCode(@Nullable final Integer code) {
			_code = code;
		}


		/**
		 * Getter method for the <LANGINFOS /> XML tag, which is a direct descendant of <COUNTRY />.
		 *
		 * @return A {@link CountryLangInfos} object.
		 */
		@InheritAnnotations
		@Mandatory
		@Override
		public CountryLangInfos getLangInfos() {
			if (_langinfos == null) {
				_langinfos = new CountryLangInfos();
			}
			return _langinfos;
		}


		/**
		 * Return the tag's label iff the {@link Country} object contains a descendant {@link CountryLangInfos} object.
		 *
		 * @param lang The language abbreviation.
		 * @return The tag's label.
		 */
		@Override
		public String label(final String lang) {
			final GomLangInfo langInfo = getLangInfos().resolve(lang);
			return langInfo != null ? langInfo.getLabel() : "";
		}


		/**
		 * Return the tag's description iff the {@link Country} object contains a descendant {@link CountryLangInfos} object.
		 *
		 * @param lang The langauge abbreviation.
		 * @return The tag's description.
		 */
		@Override
		public String description(final String lang) {
			final GomLangInfo langInfo = getLangInfos().resolve(lang);
			return langInfo != null ? langInfo.getDescription() : "";
		}


		/**
		 * Definition of the class {@link CountryLangInfos} used by the getter method {@link #getLangInfos()}.
		 */
		public static class CountryLangInfos extends DefaultGomLangInfos {

			private static final long serialVersionUID = -6815654153383100208L;

			public static final String TAG = "LANGINFOS";


			@Override
			public String getDefaultTag() {
				return TAG;
			}


			@Override
			public boolean hasRepresentation() {
				return false;
			}
		}
	}


	private static class CountryComparator implements Comparator<Country>, Serializable {

		private static final long serialVersionUID = 1L;


		/**
		 * Compare the given countries. If neither one's code is {@code null}, uses lexicographic analysis to
		 * provide greater or less than info.
		 *
		 * @param country1 The first country.
		 * @param country2 The second country.
		 * @return An indicator as to the lexicographic sort order of these {@link Country} objects.
		 */
		@Override
		public int compare(final Country country1, final Country country2) {
			final Integer code1 = country1.getCode();
			final Integer code2 = country2.getCode();
			if (code1 == null && code2 == null) {
				// Both code strings are null; we'll consider them to be alike.
				return 0;
			}
			if (code1 != null && code2 == null) {
				// First country's code is not null, while the second country's code is null; we'll consider the first country as greater than the other.
				return 1;
			}
			if (code1 == null) {
				// First country's code is null, while the second country's code is not null; we'll consider the fist country as less than the other.
				return -1;
			}
			// Both codes contain a value; perform String's lexicographic comparison to determine a return value.
			return code1.compareTo(code2);
		}

	}
}