package de.espirit.firstspirit.opt.example.universal.gadget;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.AbstractCollection;
import java.util.Collections;
import java.util.Iterator;
import java.util.NoSuchElementException;


/**
 * Implementation of the data value type specified by the interface {@link PhoneNumber}.
 *
 * This implementation assumes that a phone number consists of at least a city code and a local number, while the
 * international code is deemed optional.
 */
@SuppressWarnings("WeakerAccess")
public class PhoneNumberImpl extends AbstractCollection<Character> implements PhoneNumber {

	private Integer _internationalCode;
	private String _cityCode;
	private String _localNumber;


	/**
	 * Constructor which initializes an empty object ({@link #isEmpty()} will return {@code null}).
	 */
	PhoneNumberImpl() {
		_internationalCode = 0;
		_cityCode = "";
		_localNumber = "";
	}


	/**
	 * Constructor which initializes an object, copying the data of the provided object if available.
	 * If the {@code copy} object is {@code null}, an empty object will be initialized.
	 *
	 * @param copy The {@link PhoneNumberImpl} object to copy data from.
	 */
	PhoneNumberImpl(@Nullable final PhoneNumberImpl copy) {
		if (copy != null) {
			_internationalCode = copy.getInternationalCode();
			_cityCode = copy.getCityCode();
			_localNumber = copy.getLocalNumber();
		} else {
			_internationalCode = 0;
			_cityCode = "";
			_localNumber = "";
		}
	}


	@NotNull
	@Override
	public Integer getInternationalCode() {
		return _internationalCode;
	}


	@Override
	public void setInternationalCode(@Nullable final Integer internationalCode) {
		_internationalCode = internationalCode != null ? internationalCode : 0;
	}


	@NotNull
	@Override
	public String getCityCode() {
		return _cityCode;
	}


	@Override
	public void setCityCode(@Nullable final String cityCode) {
		_cityCode = cityCode != null ? cityCode : "";
	}


	@NotNull
	@Override
	public String getLocalNumber() {
		return _localNumber;
	}


	@Override
	public void setLocalNumber(@Nullable final String localNumber) {
		_localNumber = localNumber != null ? localNumber : "";
	}


	/**
	 * Indicate if this object is to be considered empty. This {@link PhoneNumber} implementation considers a combination of
	 * city code and local number as the smallest form of valid phone number, so this method only considers the
	 * {@link String}s {@code _cityCode} and {@code _localNumber}, returning {@code true} if at least one of them is empty.
	 *
	 * @return {@code true} if the object is considered empty.
	 */
	@Override
	public boolean isEmpty() {
		return _cityCode.isEmpty() || _localNumber.isEmpty();
	}


	@Override
	public int size() {
		return toString().length();
	}


	@NotNull
	@Override
	public Iterator<Character> iterator() {
		final String stringRepresentation = toString();
		return new Iterator<Character>() {
			int pos = -1;
			@Override
			public boolean hasNext() {
				return pos < stringRepresentation.length();
			}


			@Override
			public Character next() {
				if (hasNext()) {
					pos++;
					return stringRepresentation.charAt(pos);
				}
				throw new NoSuchElementException();
			}


			@Override
			public void remove() {
				throw new UnsupportedOperationException("remove");
			}
		};
	}


	@Override
	public int hashCode() {
		return 31 ^ _internationalCode.hashCode() ^ _cityCode.hashCode() ^ _localNumber.hashCode();
	}


	@Override
	public boolean equals(final Object object) {
		if (object instanceof PhoneNumberImpl) {
			final PhoneNumberImpl phoneNumber = (PhoneNumberImpl) object;
			return _internationalCode.equals(phoneNumber.getInternationalCode())
				&& _cityCode.equals(phoneNumber.getCityCode())
				&& _localNumber.equals(phoneNumber.getLocalNumber());
		}
		return false;
	}


	/**
	 * If this object is not considered empty, return a human-readable {@link String} representing the phone number data.
	 *
	 * @return A human-readable phone number {@link String}.
	 */
	@Override
	public String toString() {
		if (!isEmpty()) {
			if (_internationalCode != null && _internationalCode > 0) {
				return '+' + String.valueOf(_internationalCode) + " (" + _cityCode + ") " + _localNumber;
			} else {
				return '(' + _cityCode + ") " + _localNumber;
			}
		}
		return "";
	}
}
