package de.espirit.firstspirit.opt.example.universal.gadget;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.templatestore.gom.YesNo;
import de.espirit.firstspirit.agency.TransferAgent;
import de.espirit.firstspirit.aspects.AspectMap;
import de.espirit.firstspirit.aspects.AspectType;
import de.espirit.firstspirit.client.access.editor.Difference;
import de.espirit.firstspirit.client.search.SegmentProvider;
import de.espirit.firstspirit.ui.gadgets.GadgetIdentifier;
import de.espirit.firstspirit.ui.gadgets.aspects.DifferenceVisualizing;
import de.espirit.firstspirit.ui.gadgets.aspects.Editable;
import de.espirit.firstspirit.ui.gadgets.aspects.IntegrityValidating;
import de.espirit.firstspirit.ui.gadgets.aspects.Labelable;
import de.espirit.firstspirit.ui.gadgets.aspects.ValueHolder;
import de.espirit.firstspirit.ui.gadgets.aspects.focus.SwingFocusable;
import de.espirit.firstspirit.ui.gadgets.aspects.transfer.SupplierHost;
import de.espirit.firstspirit.ui.gadgets.aspects.transfer.TransferSupplying;
import de.espirit.firstspirit.ui.gadgets.swing.NotificationHost;
import de.espirit.firstspirit.ui.gadgets.swing.SwingGadget;
import de.espirit.firstspirit.ui.gadgets.swing.SwingGadgetContext;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * Implementation of a phone number Swing gadget for use in SiteArchitect.
 */
public class PhoneNumberSwingGadget implements SwingGadget, ActionListener, DocumentListener {

	private final SwingGadgetContext<GomPhoneNumber> _context;
	private final AspectMap _aspects = new AspectMap();
	private final AspectValueHolder _aspectValueHolder = new AspectValueHolder();

	private PhoneNumber _phoneNumber;
	private List<PhoneNumberUtil.GadgetCountry> _gadgetCountries;
	private final boolean _useInternationalCode;

	private boolean _editable;

	private JPanel _panel;

	@SuppressWarnings("rawtypes")
	private JComboBox _internationalCodeField;
	private JTextField _cityCodeField;
	private JTextField _localNumberField;


	public PhoneNumberSwingGadget(final SwingGadgetContext<GomPhoneNumber> context) {
		_context = context;

		// Decide if the international code Swing component should be displayed (don't display if the GOM XML doesn't define country codes).
		_useInternationalCode = PhoneNumberUtil.useInternationalCode(getForm());

		addAspect(ValueHolder.TYPE, _aspectValueHolder);
		addAspect(Editable.TYPE, new AspectEditable());
		addAspect(Labelable.TYPE, new AspectEditable());
		addAspect(SwingFocusable.TYPE, new AspectSwingFocusable());
		addAspect(DifferenceVisualizing.TYPE, new AspectDifferenceVisualizing());
		addAspect(IntegrityValidating.TYPE, new AspectIntegrityValidating());
		addAspect(TransferSupplying.TYPE, new AspectTransferSupplying());
	}


	/* SwingGadget */

	public final <A> A getAspect(@NotNull final AspectType<A> type) {
		return _aspects.get(type);
	}


	/**
	 * Generate the Swing gadget's user interface.
	 *
	 * @return A {@link JComponent} containing the gadget's user interface.
	 */
	@Override
	public JComponent getComponent() {
		if (_panel == null) {
			final TableLayout layout = new TableLayout();
			layout.setHGap(4);
			layout.setVGap(4);
			layout.setColumn(new double[] {TableLayoutConstraints.PREFERRED, 0, TableLayoutConstraints.PREFERRED, 0, TableLayoutConstraints.FILL});
			layout.setRow(new double[] {TableLayoutConstraints.PREFERRED, 0, TableLayoutConstraints.PREFERRED});

			_panel = new JPanel(layout);
			_panel.setOpaque(false);

			// Generate the Swing component related to international dialing code and add it to the panel object only if international codes are used by this instance.
			if (_useInternationalCode) {
				final JLabel internationalCodeLabel = new JLabel();
				internationalCodeLabel.setText(PhoneNumberUtil.getResourceBundleForLanguage(getDisplayLanguage()).getString("gadget.international_code"));
				_panel.add(internationalCodeLabel, "0,0");

				//noinspection unchecked
				_internationalCodeField = new JComboBox(getGadgetCountries().toArray());
				if (getForm().getRequireCountryCode() == YesNo.YES && getForm().getDefaultCountryCode() != null) {
					_internationalCodeField.setSelectedItem(PhoneNumberUtil.findCountryByCode(getGadgetCountries(), getForm().getDefaultCountryCode()));
				}
				_internationalCodeField.setEnabled(_editable);
				_internationalCodeField.addActionListener(this);
				_panel.add(_internationalCodeField, "0,2");
			}

			// Generate the Swing component related to city code and add it to the panel object.
			final JLabel cityCodeLabel = new JLabel();
			cityCodeLabel.setText(PhoneNumberUtil.getResourceBundleForLanguage(getDisplayLanguage()).getString("gadget.city_code"));
			_panel.add(cityCodeLabel, "2,0");

			_cityCodeField = new JTextField();
			_cityCodeField.setEnabled(_editable);
			_cityCodeField.getDocument().addDocumentListener(this);
			_panel.add(_cityCodeField, "2,2");

			// Generate the Swing component related to local number and add it to the panel object.
			final JLabel localNumberLabel = new JLabel();
			localNumberLabel.setText(PhoneNumberUtil.getResourceBundleForLanguage(getDisplayLanguage()).getString("gadget.local_number"));
			_panel.add(localNumberLabel, "4,0");

			_localNumberField = new JTextField();
			_localNumberField.setEnabled(_editable);
			_localNumberField.getDocument().addDocumentListener(this);
			_panel.add(_localNumberField, "4,2");
		}
		return _panel;
	}


	/**
	 * Aspect: {@link ValueHolder ValueHolder<T>}
	 */

	public class AspectValueHolder implements ValueHolder<PhoneNumber> {

		@Nullable
		@Override
		public PhoneNumber getValue() {
			final PhoneNumber phoneNumber = new PhoneNumberImpl();

			if (_internationalCodeField != null) {
				final GomPhoneNumber.Country country = ((PhoneNumberUtil.GadgetCountry) _internationalCodeField.getSelectedItem()).getCountry();
				if (country != null) {
					phoneNumber.setInternationalCode(country.getCode());
				} else {
					phoneNumber.setInternationalCode(0);
				}
			}
			phoneNumber.setCityCode(_cityCodeField.getText());
			phoneNumber.setLocalNumber(_localNumberField.getText());

			return phoneNumber;
		}


		@Override
		public void setValue(@Nullable final PhoneNumber value) {
			setPhoneNumber(value);

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					if (_useInternationalCode) {
						if (!(getPhoneNumber().getInternationalCode() > 0) && getForm().getDefaultCountryCode() != null) {
							_internationalCodeField.setSelectedItem(PhoneNumberUtil.findCountryByCode(getGadgetCountries(), getForm().getDefaultCountryCode()));
						} else if (!(getPhoneNumber().getInternationalCode() > 0)) {
							_internationalCodeField.setSelectedIndex(0);
						} else {
							_internationalCodeField.setSelectedItem(PhoneNumberUtil.findCountryByCode(getGadgetCountries(), getPhoneNumber().getInternationalCode()));
						}
					}
					_cityCodeField.setText(getPhoneNumber().getCityCode());
					_localNumberField.setText(getPhoneNumber().getLocalNumber());
				}
			});
		}


		@Override
		public boolean isEmpty() {
			return getPhoneNumber().isEmpty();
		}
	}


	/**
	 *  Aspect: {@link Editable} (also implements aspect {@link Labelable})
	 */

	public class AspectEditable implements Editable {

		@Override
		public void setEditable(final boolean editable) {
			_editable = editable;
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					if (_useInternationalCode) {
						_internationalCodeField.setEnabled(_editable);
					}
					_cityCodeField.setEnabled(_editable);
					_localNumberField.setEnabled(_editable);
				}
			});
		}


		@Override
		public final String getLabel() {
			return getForm().label(_context.getDisplayLanguage().getAbbreviation());
		}
	}


	/**
	 * Aspect: {@link SwingFocusable}
	 */

	public class AspectSwingFocusable implements SwingFocusable {

		@Override
		public void acceptFocus(@NotNull final Handler handler) {
			handler.focusOn(_panel);
		}
	}


	/**
	 * Aspect: {@link DifferenceVisualizing}
	 */

	public class AspectDifferenceVisualizing implements DifferenceVisualizing {

		@Override
		public void visualizeDifferences(final List<Difference> differences) {
			if (_useInternationalCode) {
				_internationalCodeField.setBackground(null);
			}
			_cityCodeField.setBackground(null);
			_localNumberField.setBackground(null);
			for (final Difference difference : differences) {
				final Object contrast = difference.getContrast();
				if (contrast instanceof PhoneNumber.Data) {
					final PhoneNumber.Data contrastData = (PhoneNumber.Data) contrast;
					switch (contrastData) {
						case INTERNATIONAL:
							if (_useInternationalCode) {
								_internationalCodeField.setBackground(difference.getModification().getColor());
							}
							break;
						case CITY:
							_cityCodeField.setBackground(difference.getModification().getColor());
							break;
						case LOCAL:
							_localNumberField.setBackground(difference.getModification().getColor());
							break;
						default:
							break;
					}
				}
			}
		}
	}


	/**
	 * Aspect: {@link IntegrityValidating}
	 */

	public class AspectIntegrityValidating implements IntegrityValidating {

		@NotNull
		@Override
		public Set<? extends Problem> validateIntegrity() {
			if (_editable) {
				final Set<PhoneNumberUtil.PhoneNumberProblem> problems = new HashSet<PhoneNumberUtil.PhoneNumberProblem>();

				final List<Character> allowedNumericalCharacters = Arrays.asList('0', '1', '2', '3', '4', '5', '6', '7', '8', '9');
				final List<Character> allowedSpacingCharacters = Arrays.asList(' ', '-');

				// Determine if the gadget is "empty". If so, don't return any problems so that a component without contents may be saved. Conditions:
				//  - City code field is empty
				//  - Local code field is empty
				if (_cityCodeField.getText().isEmpty() && _localNumberField.getText().isEmpty()) {
					return Collections.emptySet();
				}

				if (_useInternationalCode && getForm().getRequireCountryCode() != null && getForm().getRequireCountryCode() == YesNo.YES) {
					Integer internationalCode = null;
					if (_internationalCodeField != null) {
						final PhoneNumberUtil.GadgetCountry selectedItem = (PhoneNumberUtil.GadgetCountry) _internationalCodeField.getSelectedItem();
						if (selectedItem.getCountry() == null) {
							internationalCode = 0;
						} else {
							internationalCode = selectedItem.getCountry().getCode();
						}
					}
					if (internationalCode == null || !(internationalCode > 0)) {
						final PhoneNumberUtil.PhoneNumberProblem problem = new PhoneNumberUtil.PhoneNumberProblem("gadget.problem.empty.international_code", getDisplayLanguage());
						problems.add(problem);
					}
				}

				if (_cityCodeField.getText().isEmpty()) {
					final PhoneNumberUtil.PhoneNumberProblem problem = new PhoneNumberUtil.PhoneNumberProblem("gadget.problem.empty.city_code", getDisplayLanguage());
					problems.add(problem);
				} else {
					for (final Character character : _cityCodeField.getText().toCharArray()) {
						if (!allowedNumericalCharacters.contains(character)) {
							final PhoneNumberUtil.PhoneNumberProblem problem = new PhoneNumberUtil.PhoneNumberProblem("gadget.problem.invalid_format.city_code.numeric", getDisplayLanguage());
							problems.add(problem);
						}
					}
				}

				if (_localNumberField.getText().isEmpty()) {
					final PhoneNumberUtil.PhoneNumberProblem problem = new PhoneNumberUtil.PhoneNumberProblem("gadget.problem.empty.local_number", getDisplayLanguage());
					problems.add(problem);
				} else {
					for (final Character character : _localNumberField.getText().toCharArray()) {
						if (!allowedNumericalCharacters.contains(character) && !allowedSpacingCharacters.contains(character)) {
							final PhoneNumberUtil.PhoneNumberProblem problem = new PhoneNumberUtil.PhoneNumberProblem("gadget.problem.invalid_format.local_number", getDisplayLanguage());
							problems.add(problem);
						}
					}
				}

				return problems;
			}
			return Collections.emptySet();
		}
	}


	/**
	 * Aspect: {@link TransferSupplying}
	 */

	public class AspectTransferSupplying implements TransferSupplying {

		@Override
		public void registerSuppliers(@NotNull final SupplierHost host) {
			final TransferAgent transferAgent = _context.getBroker().requireSpecialist(TransferAgent.TYPE);

			// Create and register a Supplier object which supplies a PhoneNumber object.
			final SupplierHost.Supplier<PhoneNumber> phoneNumberSupplier = new SupplierHost.Supplier<PhoneNumber>() {

				@Override
				public List<PhoneNumber> supply() {
					return Collections.singletonList(_aspectValueHolder.getValue());
				}
			};
			host.register(transferAgent.getRawValueType(PhoneNumber.class), phoneNumberSupplier);

			// Create and register a Supplier object which supplies a String containing a human-readable phone number.
			final SupplierHost.Supplier<String> stringSupplier = new SupplierHost.Supplier<String>() {
				@Override
				public List<String> supply() {
					final PhoneNumber phoneNumber = _aspectValueHolder.getValue();
					if (phoneNumber != null && !phoneNumber.isEmpty()) {
						return Collections.singletonList(phoneNumber.toString());
					}
					return Collections.emptyList();
				}
			};
			host.register(transferAgent.getRawValueToStringType(), stringSupplier);

			// Create and register a Supplier object which supplies a SegmentProvider object used to provide a query segment for use with the QueryAgent.
			final SupplierHost.Supplier<SegmentProvider> querySupplier = new SupplierHost.Supplier<SegmentProvider>() {
				@Override
				public List<SegmentProvider> supply() {
					final PhoneNumber phoneNumber = _aspectValueHolder.getValue();
					final SegmentProvider segmentProvider = new SegmentProvider() {
						@Nullable
						@Override
						public String getSegment() {
							return getForm().getName().toString() + " = '" + (phoneNumber != null ? phoneNumber.toString() : "") + '\'';
						}
					};
					return Collections.singletonList(segmentProvider);
				}
			};
			host.register(transferAgent.getQuerySegmentType(), querySupplier);
		}
	}


	/* ActionListener */

	@Override
	public void actionPerformed(final ActionEvent e) {
		notifyValueChange(getGadgetID());
	}


	/* DocumentListener */

	@Override
	public void insertUpdate(final DocumentEvent e) {
		notifyValueChange(getGadgetID());
	}


	@Override
	public void removeUpdate(final DocumentEvent e) {
		notifyValueChange(getGadgetID());
	}


	@Override
	public void changedUpdate(final DocumentEvent e) {
		notifyValueChange(getGadgetID());
	}


	/* Convenience methods */

	private <A> void addAspect(@NotNull final AspectType<A> type, @NotNull final A aspect) {
		_aspects.put(type, aspect);
	}


	@NotNull
	private GomPhoneNumber getForm() {
		return _context.getGom();
	}


	@NotNull
	private NotificationHost getNotificationHost() {
		return _context.getNotificationHost();
	}


	@NotNull
	private Language getDisplayLanguage() {
		return _context.getDisplayLanguage();
	}


	@NotNull
	private GadgetIdentifier getGadgetID() {
		return GadgetIdentifier.from(getForm());
	}


	private void notifyValueChange(final GadgetIdentifier identifier) {
		getNotificationHost().onModification(identifier);
	}


	@NotNull
	private PhoneNumber getPhoneNumber() {
		if (_phoneNumber == null) {
			_phoneNumber = new PhoneNumberImpl();
		}
		return _phoneNumber;
	}


	private void setPhoneNumber(@Nullable final PhoneNumber phoneNumber) {
		_phoneNumber = new PhoneNumberImpl((PhoneNumberImpl) phoneNumber);
	}


	/**
	 * Helper method to obtain the country codes specified in GOM XML based on the persistency language, wrapped in a
	 * {@link PhoneNumberUtil.GadgetCountry} object. This is for use with the {@link JComboBox} {@link #_internationalCodeField},
	 * because GOM elements (such as {@link GomPhoneNumber.Country} are agnostic of FirstSpirit's project language model.
	 *
	 * @return A {@link List} of {@link PhoneNumberUtil.GadgetCountry} objects.
	 */
	private List<PhoneNumberUtil.GadgetCountry> getGadgetCountries() {
		if (_gadgetCountries == null) {
			_gadgetCountries = new ArrayList<PhoneNumberUtil.GadgetCountry>();
			_gadgetCountries.addAll(PhoneNumberUtil.getGadgetCountries(getForm().getCountries(), getForm().getRequireCountryCode(), _context.getPersistencyLanguage()));
		}
		return _gadgetCountries;
	}
}
