package de.espirit.firstspirit.opt.example.universal.gadget;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.templatestore.gom.YesNo;
import de.espirit.firstspirit.ui.gadgets.aspects.IntegrityValidating;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JComboBox;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;


/**
 * Utility class for the phone number input component example. This class contains various useful methods and classes
 * which are used throughout other classes of this code example.
 */
public class PhoneNumberUtil {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.universal.gadget.PhoneNumberMessages";


	/**
	 * Obtain and return the phone number component's resource bundle for the given language.
	 *
	 * @param language The {@link Language} to obtain a resource bundle for.
	 * @return The {@link ResourceBundle}.
	 */
	public static ResourceBundle getResourceBundleForLanguage(@NotNull final Language language) {
		return ResourceBundle.getBundle(MESSAGE_BUNDLE, language.getLocale());
	}


	/**
	 * Decide and indicate if processing of an international code is required, depending on the input component's
	 * configuration in a template's form definition.
	 *
	 * @param form The input component configuration as given in a template's form definition.
	 * @return {@code true} if an international code should be considered, {@code false} otherwise.
	 */
	public static boolean useInternationalCode(@NotNull final GomPhoneNumber form) {
		return form.getCountries() != null && !form.getCountries().isEmpty();
	}


	/**
	 * Compile a list of {@link GadgetCountry} objects for use with the {@link PhoneNumberSwingGadget}'s international code {@link JComboBox}.
	 * If it is indicated, that country codes are not required, the list will also contain an empty {@code GadgetCountry} object as a placeholder.
	 *
	 * @param countries A {@link GomPhoneNumber.Countries} object containing {@link GomPhoneNumber.Country} objects to process.
	 * @param requireCountryCodes Are country codes required?
	 * @param persistenceLanguage The {@link Language} for which a human-readable text (including country name) should be generated.
	 * @return A {@link List} of {@link GadgetCountry} objects.
	 */
	public static List<GadgetCountry> getGadgetCountries(@NotNull final GomPhoneNumber.Countries countries, @Nullable final YesNo requireCountryCodes, @NotNull final Language persistenceLanguage) {
		final List<GadgetCountry> gadgetCountries = new ArrayList<GadgetCountry>();

		if (requireCountryCodes == null || requireCountryCodes == YesNo.NO) {
			// Add an empty GadgetCountry object as a default item for comboboxes.
			gadgetCountries.add(new GadgetCountry());
		}

		// Add all Country objects configured for the currently processed Countries object.
		for (final GomPhoneNumber.Country country : countries) {
			gadgetCountries.add(new GadgetCountry(country, persistenceLanguage));
		}

		return gadgetCountries;
	}


	/**
	 * From a {@link List} of {@link GadgetCountry} objects, identify a single {@code GadgetCountry} matching a given code.
	 *
	 * @param gadgetCountries A {@link List} of {@link GadgetCountry} objects.
	 * @param code A country code to search for.
	 * @return A {@link GadgetCountry} object.
	 */
	public static GadgetCountry findCountryByCode(@NotNull final List<GadgetCountry> gadgetCountries, @Nullable final Integer code) {
		for (final GadgetCountry gadgetCountry : gadgetCountries) {
			final GomPhoneNumber.Country country = gadgetCountry.getCountry();
			if (country != null && country.getCode() != null && country.getCode().equals(code)) {
				return gadgetCountry;
			}
		}
		return gadgetCountries.get(0);
	}


	/**
	 * Adapter object providing information about a {@link GomPhoneNumber.Country} object in a format which produces a
	 * language-specific {@link String} form, e.g. for use in a {@link JComboBox} which should be internationalized.
	 */
	public static class GadgetCountry {

		private final GomPhoneNumber.Country _country;
		private final Language _persistenceLanguage;


		public GadgetCountry() {
			_country = null;
			_persistenceLanguage = null;
		}


		public GadgetCountry(@NotNull final GomPhoneNumber.Country country, @NotNull final Language persistenceLanguage) {
			_country = country;
			_persistenceLanguage = persistenceLanguage;
		}


		@Nullable
		public GomPhoneNumber.Country getCountry() {
			return _country != null ? _country : new GomPhoneNumber.Country();
		}


		@NotNull
		public String toString() {
			return _country != null ? (_country.getCode() + " - " + _country.label(_persistenceLanguage.getAbbreviation())) : "";
		}
	}


	/**
	 * {@link IntegrityValidating.Problem} implementation for use by {@link PhoneNumberSwingGadget.AspectIntegrityValidating}.
	 */
	public static class PhoneNumberProblem implements IntegrityValidating.Problem {

		private final String _resourceKey;
		private final Language _displayLanguage;


		public PhoneNumberProblem(@NotNull final String resourceKey, @NotNull final Language displayLanguage) {
			_resourceKey = resourceKey;
			_displayLanguage = displayLanguage;
		}


		@Override
		public String getMessageCode() {
			return getResourceBundleForLanguage(_displayLanguage).getString(_resourceKey);
		}


		@Override
		public Map<String, Object> getMessageContext() {
			return Collections.emptyMap();
		}
	}
}
