package de.espirit.firstspirit.opt.example.universal.gadget;

import de.espirit.firstspirit.access.editor.ValueIndexer;
import de.espirit.firstspirit.access.editor.value.Modification;
import de.espirit.firstspirit.client.access.editor.Difference;
import de.espirit.firstspirit.client.access.editor.DifferenceComputing;
import de.espirit.firstspirit.client.access.editor.Node;
import de.espirit.firstspirit.client.access.editor.ReferenceAwareValueIndexSupporting;
import de.espirit.firstspirit.client.access.editor.ValueEngineer;
import de.espirit.firstspirit.client.access.editor.ValueEngineerAspectType;
import de.espirit.firstspirit.client.access.editor.ValueEngineerContext;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Implementation of a {@link ValueEngineer} for the phone number example input component.
 */
public class PhoneNumberValueEngineer implements ValueEngineer<PhoneNumber> {

	private final ValueEngineerContext<GomPhoneNumber> _context;

	private final Map<ValueEngineerAspectType<?>, Object> _aspects = new HashMap<ValueEngineerAspectType<?>, Object>();

	private final boolean _useInternationalCode;


	public PhoneNumberValueEngineer(@NotNull final ValueEngineerContext<GomPhoneNumber> context) {
		_context = context;

		// Decide if the international code should be used in value processing (don't process if the GOM XML doesn't define country codes).
		_useInternationalCode = PhoneNumberUtil.useInternationalCode(_context.getGom());

		addAspect(DifferenceComputing.TYPE, new AspectDifferenceComputing());
		addAspect(ReferenceAwareValueIndexSupporting.TYPE, new AspectReferenceAwareValueIndexSupporting());
	}


	/**
	 * Provide a {@link Node}-based form of serializable data in order to store the input component's value.
	 *
	 * @param value The value to write as list of nodes.
	 * @return A {@link List} of {@link Node} objects.
	 */
	@NotNull
	@Override
	public List<Node> write(@NotNull final PhoneNumber value) {
		final List<Node> nodes = new ArrayList<Node>();

		if (_useInternationalCode) {
			final Integer internationalCode = value.getInternationalCode();
			nodes.add(Node.create(PhoneNumber.Data.INTERNATIONAL.name(), String.valueOf(internationalCode)));
		}

		final String cityCode = value.getCityCode();
		nodes.add(Node.create(PhoneNumber.Data.CITY.name(), cityCode));

		final String localNumber = value.getLocalNumber();
		nodes.add(Node.create(PhoneNumber.Data.LOCAL.name(), localNumber));

		return nodes;
	}


	/**
	 * Provide a {@link PhoneNumber} object assembled from the {@link Node} data provided.
	 *
	 * @param nodes The nodes representing a value.
	 * @return A {@link PhoneNumber} object.
	 */
	@Nullable
	@Override
	public PhoneNumber read(@NotNull final List<Node> nodes) {
		final PhoneNumberImpl phoneNumber = new PhoneNumberImpl();
		for (final Node node : nodes) {
			final String nodeName = node.getName();

			if (PhoneNumber.Data.INTERNATIONAL.name().equals(nodeName)) {
				phoneNumber.setInternationalCode(Integer.valueOf(node.getText()));
			}

			if (PhoneNumber.Data.CITY.name().equals(nodeName)) {
				phoneNumber.setCityCode(node.getText());
			}

			if (PhoneNumber.Data.LOCAL.name().equals(nodeName)) {
				phoneNumber.setLocalNumber(node.getText());
			}
		}
		return phoneNumber;
	}


	/**
	 * Provide an empty {@link PhoneNumber} object.
	 *
	 * @return A {@link PhoneNumber} object.
	 */
	@Nullable
	@Override
	public PhoneNumber getEmpty() {
		return new PhoneNumberImpl();
	}


	/**
	 * Indicate if the provided {@link PhoneNumber} object is considered empty.
	 *
	 * @param value Value to check.
	 * @return {@code true} if the object is empty.
	 */
	@Override
	public boolean isEmpty(@NotNull final PhoneNumber value) {
		return value.isEmpty();
	}


	/**
	 * Assemble a new {@link PhoneNumber} object based on data of a given {@code PhoneNumber} object.
	 *
	 * @param original The value to be copied.
	 * @return A new {@link PhoneNumber} object containing the same data.
	 */
	@NotNull
	@Override
	public PhoneNumber copy(@NotNull final PhoneNumber original) {
		final PhoneNumberImpl copy = new PhoneNumberImpl();
		copy.setInternationalCode(original.getInternationalCode());
		copy.setCityCode(original.getCityCode());
		copy.setLocalNumber(original.getLocalNumber());
		return copy;
	}


	@Nullable
	@Override
	public <A> A getAspect(@NotNull final ValueEngineerAspectType<A> aspect) {
		return aspect.cast(_aspects.get(aspect));
	}


	public <A> void addAspect(@NotNull final ValueEngineerAspectType<A> type, @NotNull final A aspect) {
		_aspects.put(type, aspect);
	}


	/**
	 * Aspect: {@link DifferenceComputing DifferenceComputing<T>}
	 */
	public static class AspectDifferenceComputing implements DifferenceComputing<PhoneNumber> {

		@NotNull
		@Override
		public List<Difference> computeDifferences(@NotNull final PhoneNumber actualValue, @NotNull final PhoneNumber oldValue) {
			final List<Difference> differences = new ArrayList<Difference>();
			if (!actualValue.equals(oldValue)) {
				{
					final Integer actualInternationalCode = actualValue.getInternationalCode();
					final Integer oldInternationalCode = oldValue.getInternationalCode();

					Modification internationalModification = Modification.NONE;

					if (actualInternationalCode > 0 && !(oldInternationalCode > 0)) {
						internationalModification = Modification.INSERTED;
					} else if (!(actualInternationalCode > 0) && oldInternationalCode > 0) {
						internationalModification = Modification.DELETED;
					} else if (!actualInternationalCode.equals(oldInternationalCode)) {
						internationalModification = Modification.CHANGED;
					}

					differences.add(new Difference(PhoneNumber.Data.INTERNATIONAL, internationalModification));
				}
				{
					final String actualCityCode = actualValue.getCityCode();
					final String oldCityCode = oldValue.getCityCode();

					Modification cityModification = Modification.NONE;

					if (!actualCityCode.isEmpty() && oldCityCode.isEmpty()) {
						cityModification = Modification.INSERTED;
					} else if (actualCityCode.isEmpty() && !oldCityCode.isEmpty()) {
						cityModification = Modification.DELETED;
					} else if (!actualCityCode.equals(oldCityCode)) {
						cityModification = Modification.CHANGED;
					}

					differences.add(new Difference(PhoneNumber.Data.CITY, cityModification));
				}
				{
					final String actualLocalNumber = actualValue.getLocalNumber();
					final String oldLocalNumber = oldValue.getLocalNumber();

					Modification localModification = Modification.NONE;

					if (!actualLocalNumber.isEmpty() && oldLocalNumber.isEmpty()) {
						localModification = Modification.INSERTED;
					} else if (actualLocalNumber.isEmpty() && !oldLocalNumber.isEmpty()) {
						localModification = Modification.DELETED;
					} else if (!actualLocalNumber.equals(oldLocalNumber)) {
						localModification = Modification.CHANGED;
					}

					differences.add(new Difference(PhoneNumber.Data.LOCAL, localModification));
				}
			}
			return differences;
		}
	}


	/**
	 * Aspect: {@link ReferenceAwareValueIndexSupporting ReferenceAwareValueIndexSupporting<T>}
	 *
	 * This class appends both the input component's name ("name" attribute in a template's GOM form definition) and the
	 * phone number's String representation so that search queries of the format "key = value" are supported.
	 */
	public class AspectReferenceAwareValueIndexSupporting implements ReferenceAwareValueIndexSupporting<PhoneNumber> {

		@Override
		public void appendIndexData(@NotNull final ValueIndexer indexer, @NotNull final PhoneNumber value, final boolean includeReferences) {
			// Set the key field to the input component's name (attribute "name").
			indexer.append(ValueIndexer.KEY_FIELD, _context.getGom().getName().toString());
			// Index the phone number's String value.
			indexer.append(ValueIndexer.VALUE_FIELD, value.toString());
		}
	}
}
