package de.espirit.firstspirit.opt.example.universal.gadget;

import de.espirit.common.tools.Streams;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.templatestore.gom.YesNo;
import de.espirit.firstspirit.aspects.AspectMap;
import de.espirit.firstspirit.aspects.AspectType;
import de.espirit.firstspirit.module.GadgetContext;
import de.espirit.firstspirit.webedit.server.gadgets.WebPluginGadget;
import de.espirit.firstspirit.webedit.server.gadgets.aspects.SerializingValueHolder;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;


/**
 * Implementation of server-side functionality for a phone number web gadget for use in ContentCreator.
 */
public class PhoneNumberWebGadget implements WebPluginGadget<HashMap<String, Serializable>>, SerializingValueHolder<PhoneNumber, HashMap<String, Serializable>> {

	private final GadgetContext<GomPhoneNumber> _context;
	private final AspectMap _aspects;
	private final Language _persistenceLanguage;
	private final boolean _useInternationalCode;

	private PhoneNumber _phoneNumber;


	public PhoneNumberWebGadget(final GadgetContext<GomPhoneNumber> context) {
		_context = context;
		_persistenceLanguage = _context.getPersistencyLanguage();

		// Decide if the international code form input field should be displayed (don't display if the GOM XML doesn't define country codes).
		_useInternationalCode = PhoneNumberUtil.useInternationalCode(_context.getGom());

		_aspects = new AspectMap();
		_aspects.put(SerializingValueHolder.TYPE, this);
	}


	/**
	 * Create and populate a configuration {@link Map} which may be used to provide useful input component data to the
	 * JavaScript controller object used to implement client-side gadget functionality.
	 *
	 * @return A {@link Map} of serializable gadget configuration data.
	 */
	@Override
	public HashMap<String, Serializable> getWidgetConfiguration() {
		final HashMap<String, Serializable> configuration = new HashMap<String, Serializable>();
		configuration.put("label", _context.getGom().label(_persistenceLanguage.getAbbreviation()));
		configuration.put("offerInternationalCode", _useInternationalCode);
		configuration.put("requireInternationalCode", _context.getGom().getRequireCountryCode() == YesNo.YES);
		if (_context.getGom().getCountries() != null) {
			configuration.put("internationalCodes", _context.getGom().getCountries().toJSON());
		}
		configuration.put("formLanguage", _persistenceLanguage.getAbbreviation());
		if (_context.getGom().getDefaultCountryCode() != null) {
			configuration.put("defaultCountryCode", _context.getGom().getDefaultCountryCode());
		}
		return configuration;
	}


	@Nullable
	@Override
	public <T> T getAspect(@NotNull final AspectType<T> aspect) {
		return _aspects.get(aspect);
	}


	@Nullable
	@Override
	public PhoneNumber getValue() {
		return _phoneNumber;
	}


	@Override
	public void setValue(@Nullable final PhoneNumber value) {
		_phoneNumber = value;
	}


	@Override
	public boolean isEmpty() {
		return _phoneNumber == null || _phoneNumber.isEmpty();
	}


	/**
	 * Produce a {@link Map} of serializable data which represents the server-side gadget's current data value. This
	 * object will be used to provide the current data value to the client-side JavaScript controller.
	 *
	 * @return A {@link Map} of serializable data.
	 */
	@Nullable
	@Override
	public HashMap<String, Serializable> getSerializedValue() {
		final HashMap<String, Serializable> serialized = new HashMap<String, Serializable>();
		if (_phoneNumber != null) {
			serialized.put(PhoneNumber.Data.INTERNATIONAL.name(), _phoneNumber.getInternationalCode());
			serialized.put(PhoneNumber.Data.CITY.name(), _phoneNumber.getCityCode());
			serialized.put(PhoneNumber.Data.LOCAL.name(), _phoneNumber.getLocalNumber());
		}
		return serialized;
	}


	/**
	 * Set the server-side gadget's data value according to the provided {@link Map} of serialized data. This method is
	 * called to transfer data from the client-side JavaScript controller to the server-side gadget object (e.g. as a
	 * user makes edits on the input component's data in an editable form dialog).
	 *
	 * @param serialization A {@link Map} of serialized data.
	 */
	@Override
	public void setSerializedValue(@Nullable final HashMap<String, Serializable> serialization) {
		_phoneNumber = new PhoneNumberImpl();
		if (serialization != null) {
			final Integer internationalCode = Integer.valueOf((String) serialization.get(PhoneNumber.Data.INTERNATIONAL.name()));
			if (internationalCode > 0) {
				_phoneNumber.setInternationalCode(internationalCode);
			} else {
				_phoneNumber.setInternationalCode(null);
			}

			_phoneNumber.setCityCode((String) serialization.get(PhoneNumber.Data.CITY.name()));
			_phoneNumber.setLocalNumber((String) serialization.get(PhoneNumber.Data.LOCAL.name()));
		}
	}


	/**
	 * Provide the HTML code required to build the gadget's user interface in an input form.
	 *
	 * @return A {@link String} containing HTML mark-up.
	 */
	@Override
	public String getView() {
		final String phoneNumberHTML = getResource("PhoneNumberWebGadget.html");
		if (phoneNumberHTML != null && !phoneNumberHTML.isEmpty()) {
			return phoneNumberHTML;
		}
		return "";
	}


	/**
	 * Helper method which reads text data from a given file included as a Java resource.
	 *
	 * @param filename The name of the file to read from.
	 * @return A {@link String} containing the file's data.
	 */
	private static String getResource(final String filename) {
		try {
			final InputStream inputStream = PhoneNumberWebGadget.class.getResourceAsStream(filename);
			if (inputStream != null) {
				return Streams.toString(inputStream, "UTF-8");
			}
		} catch (final IOException e) {
			// Ignored.
		}
		return "";
	}
}
