package de.espirit.firstspirit.opt.example.universal.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.client.plugin.report.DataRenderer;
import de.espirit.firstspirit.client.plugin.report.Image;

import org.jetbrains.annotations.Nullable;


/**
 * Defines a renderer component that operates on {@link TextBlocksObject}s in order to provide {@link String}s for
 * display in the WebClient report plug-in.
 */
public class TextBlocksDataRenderer implements DataRenderer<TextBlocksObject> {

	private final BaseContext _context;


	/**
	 * Constructor. Stores the context as a field.
	 *
	 * @param context The context the report operates in.
	 */
	TextBlocksDataRenderer(final BaseContext context) {
		_context = context;
	}


	@Nullable
	@Override
	public Image<?> getIcon(final TextBlocksObject object) {
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// Generate a URL part for the icon file used in WebClient. The file is copied from this package's path during the Ant build process (see build.xml)!
			return Image.Factory.fromUrl("webclient_examples/media/report/textblocks/reportbutton_berry.png");
		} else {
			// The icon file used for the JavaClient is stored in this package's path and included in the JAR file generated during the Ant build process.
			return Image.Factory.fromIcon(new javax.swing.ImageIcon(TextBlocksDataRenderer.class.getResource("reportbutton_berry.png")));
		}
	}


	/**
	 * Returns the thumbnail path stored in {@code object}.
	 *
	 * @param object The object that is to be rendered in the report list.
	 * @return The thumbnail file path.
	 */
	@Override
	public Image<?> getThumbnail(final TextBlocksObject object) {
		final String imageFileName = object.getImageFileName();
		if (imageFileName != null && !imageFileName.isEmpty()) {
			if (_context.is(BaseContext.Env.WEBEDIT)) {
				// Generate URL parts for thumbnail files used in WebClient. The files are copied from this package's path during the Ant build process (see build.xml)!
				return Image.Factory.fromUrl("webclient_examples/media/report/textblocks/" + imageFileName);
			} else {
				// The thumbnail files used for the JavaClient are stored in this package's path and included in the JAR file generated during the Ant build process.
				return Image.Factory.fromIcon(new javax.swing.ImageIcon(TextBlocksDataRenderer.class.getResource(imageFileName)));
			}
		}
		return null;
	}


	/**
	 * Returns the title text to be shown for {@code object}.
	 *
	 * @param object The object that is to be rendered in the report list.
	 * @return A title text.
	 */
	@Override
	public String getTitle(final TextBlocksObject object) {
		return object.getTitle();
	}


	/**
	 * Returns the descriptive text of {@code object}.
	 *
	 * @param object The object that is to be rendered in the report list.
	 * @return A descriptive text.
	 */
	@Override
	public String getDescription(final TextBlocksObject object) {
		return object.getText();
	}


	@Nullable
	@Override
	public String getDetails(final TextBlocksObject object) {
		@SuppressWarnings("StringBufferReplaceableByString")
		final StringBuilder detailString = new StringBuilder();
		detailString.append("<section style=").append('"').append("max-width: 240px;").append('"').append('>');
		detailString.append("<img src=").append('"').append("webclient_examples/media/report/textblocks/").append(object.getImageFileName()).append('"').append(" style=").append('"').append("float: left; max-width: 29px;").append('"').append("/>");
		detailString.append("<h1 style=").append('"').append("margin-left: 49px; margin-top: 0;").append('"').append('>');
		detailString.append(object.getTitle());
		detailString.append("</h1>");
		detailString.append("<p>");
		detailString.append(object.getText());
		detailString.append("</p>");
		detailString.append("</section>");
		return detailString.toString();
	}
}
