package de.espirit.firstspirit.opt.example.universal.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.agency.UIAgent;
import de.espirit.firstspirit.client.plugin.ReportPlugin;
import de.espirit.firstspirit.client.plugin.report.DataProvider;
import de.espirit.firstspirit.client.plugin.report.DataRenderer;
import de.espirit.firstspirit.client.plugin.report.Image;
import de.espirit.firstspirit.client.plugin.report.Parameter;
import de.espirit.firstspirit.client.plugin.report.ReportItem;
import de.espirit.firstspirit.client.plugin.report.TransferHandler;
import de.espirit.firstspirit.webedit.WebeditUiAgent;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.ResourceBundle;


/**
 * Defines a WebClient report plug-in that uses {@link TextBlocksObject}s in order to display a list of example texts.
 */
public class TextBlocksReportPlugin implements ReportPlugin<TextBlocksObject> {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.universal.report.ReportMessages";

	private BaseContext _context;
	private ResourceBundle _resourceBundle;
	private TextBlocksParameters _parameters;


	@Override
	public boolean isVisible() {
		return true;
	}


	/**
	 * Provides the report's name that will be used in several locations in the WebClient user interface as well as in
	 * Server and Project Configuration.
	 *
	 * @return A short, human-readable name to identify this report.
	 */
	@Override
	public String getTitle() {
		return _resourceBundle.getString("TextBlocksReportPlugin.name");
	}


	/**
	 * Provides a path to an icon representing this report. The parameter {@code selected} is observed in order to provide
	 * distinct URLs for "inactive" and "active" icons, relative to the project-specific WebClient web application's
	 * directory.
	 *
	 * @param selected The selected-state of the icon.
	 * @return The relative URL to the report's icon, based on the WebClient web app's directory.
	 */
	@Override
	public Image<?> getIcon(final boolean selected) {
		if (_context.is(BaseContext.Env.WEBEDIT)) {
			// Generate URL parts for icon files used in WebClient. The files are copied from this package's path during the Ant build process (see build.xml)!
			if (selected) {
				return Image.Factory.fromUrl("webclient_examples/media/report/textblocks/reportbutton_berry.png");
			} else {
				return Image.Factory.fromUrl("webclient_examples/media/report/textblocks/reportbutton_grey_light.png");
			}
		} else {
			// The icon files used for the JavaClient are stored in this package's path and included in the JAR file generated during the Ant build process.
			if (selected) {
				return Image.Factory.fromIcon(new javax.swing.ImageIcon(TextBlocksReportPlugin.class.getResource("reportbutton_grey_light.png")));
			} else {
				return Image.Factory.fromIcon(new javax.swing.ImageIcon(TextBlocksReportPlugin.class.getResource("reportbutton_grey_dark.png")));
			}
		}
	}


	/**
	 * Compile a set of parameters that will be rendered in the report's filter area so the user may limit the results this
	 * report displays.
	 *
	 * @return A {@link List} containing the text, boolean and select pattern constants configured in {@code setUp()}.
	 */
	@NotNull
	@Override
	public List<Parameter<?>> getParameter() {
		return _parameters.toList();
	}


	/**
	 * Instantiate a data provider ({@link TextBlocksDataProvider}) and return this object.
	 *
	 * @return An instance of {@link TextBlocksDataProvider}.
	 */
	@NotNull
	@Override
	public DataProvider<TextBlocksObject> createDataProvider() {
		return new TextBlocksDataProvider(_context, _resourceBundle, _parameters);
	}


	/**
	 * Instantiate a renderer ({@link TextBlocksDataRenderer}) and return this object.
	 *
	 * @return An instance of {@link TextBlocksDataRenderer}.
	 */
	@NotNull
	@Override
	public DataRenderer<TextBlocksObject> createDataRenderer() {
		return new TextBlocksDataRenderer(_context);
	}


	@Override
	public TransferHandler<TextBlocksObject> createTransferHandler() {
		return new TextBlocksTransferHandler(_context);
	}


	@Nullable
	@Override
	public ReportItem<TextBlocksObject> getDefaultItem() {
		return new TextBlocksExecutableReportItem(_resourceBundle);
	}


	@NotNull
	@Override
	public Collection<? extends ReportItem<TextBlocksObject>> getItems() {
		return Collections.singletonList(new TextBlocksClientScriptProvidingReportItem(_resourceBundle));
	}


	/**
	 * Setup tasks: configure a resource bundle with the WebClient user interface's current locale and specify several
	 * filter parameters with localization.
	 *
	 * @param context The context the plug-in operates in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;

		// Determine display language (with JavaClient/WebClient fork) and obtain the resource bundle for this package.
		final Language displayLanguage;
		if (context.is(BaseContext.Env.WEBEDIT)) {
			displayLanguage = context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage();
		} else {
			displayLanguage = context.requireSpecialist(UIAgent.TYPE).getDisplayLanguage();
		}
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, displayLanguage.getLocale());
		_parameters = new TextBlocksParameters(_resourceBundle);
	}


	/**
	 * Clean-up method, used in case an object instance of this class is no longer needed.
	 */
	@Override
	public void tearDown() {
	}
}
