package de.espirit.firstspirit.opt.example.universal.timelineprovider;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.client.plugin.timeline.HistoryTimelineProvider;
import de.espirit.firstspirit.client.plugin.timeline.TimelineEntry;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;


/**
 * This {@link HistoryTimelineProvider} provides timeline markers for past dates. Specifically, this component does not
 * analyze any store element content but provides points for the previous six days as well as the previous
 * four weeks (i.e., yesterday, day before yesterday, etc. and last week, two weeks ago, etc.).
 */

public class DaysWeeksHistoryTimelineProvider implements HistoryTimelineProvider {

	private static final long MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000;

	@Override
	public void setUp(@NotNull final BaseContext context) {
	}


	@Override
	public void tearDown() {
	}


	/**
	 * Set the timeline range to 30 days.
	 */

	@Override
	public long getRange() {
		return 30 * MILLISECONDS_PER_DAY;
	}


	/**
	 * Provides timeline entries which will be shown as history markers on the Multi-Perspective Preview timeline. These
	 * entries should be located between the current time and (current time - {@code getRange()}).
	 * @param element The element for which to calculate timeline entries.
	 * @return A {@link Collection} of {@link TimelineEntry} objects.
	 */

	@NotNull
	@Override
	public Collection<TimelineEntry> getEntries(@NotNull final IDProvider element, @NotNull final Language language) {
		final long unixTimeNow = System.currentTimeMillis();
		final Collection<TimelineEntry> futureEntries = new ArrayList<TimelineEntry>();
		// Produce entries for the upcoming six days (i.e. 24 hours from now, 48 hours from now, etc.).
		for (int i = 1; i < 7; i++) {
			final TimelineEntry dailyEntry = TimelineEntry.Factory.create(new Date(unixTimeNow - (i * MILLISECONDS_PER_DAY)));
			futureEntries.add(dailyEntry);
		}
		// Produce entries for the upcoming four weeks (i.e. 7 days from now, 14 days from now, etc.).
		for (int i = 1; i < 5; i++) {
			final TimelineEntry weeklyEntry = TimelineEntry.Factory.create(new Date(unixTimeNow - (i * 7 * MILLISECONDS_PER_DAY)));
			futureEntries.add(weeklyEntry);
		}
		return futureEntries;
	}
}
