package de.espirit.firstspirit.opt.example.webedit.elementstatusprovider;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.Task;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.sitestore.Content2Params;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.access.store.templatestore.Workflow;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.workflow.WebeditElementStatusProviderPlugin;
import de.espirit.firstspirit.workflow.WorkflowGroup;
import de.espirit.or.schema.Entity;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;


/**
 * Assembles groups of elements based on datasets for use on content projection pages. Also provides rudimentary
 * handling of page references and pages for other web site parts that do not use content projection.
 */
public class ContentProjectionElementStatusProviderPlugin implements WebeditElementStatusProviderPlugin {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.elementstatusprovider.ElementStatusProviderMessages";

	private BaseContext _context;
	private ResourceBundle _resourceBundle;


	/**
	 * This enumeration specifies two group definitions - RELEASED and MODIFIED - that will be used in localization to
	 * generate display names.
	 */
	enum Group {
		RELEASED("ContentProjectionElementStatusProviderPlugin.group.released"),
		MODIFIED("ContentProjectionElementStatusProviderPlugin.group.modified");

		private final String _displayName;


		private Group(final String displayName) {
			_displayName = displayName;
		}


		public String getDisplayName() {
			return _displayName;
		}
	}


	/**
	 * Simple setup method. Store the context as a local field and configure a resource bundle with the WebClient UI
	 * language's locale.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
	}


	/**
	 * Deconstructor method.
	 * <p/>
	 * This class does not create all too complex data structures. Hence, this method does nothing.
	 */
	@Override
	public void tearDown() {
	}


	/**
	 * This method takes over main assembly of different workflow groups and returns these as a list.
	 *
	 * @param element {@link PageRef} or {@link Dataset} object.
	 * @return A list of {@link WorkflowGroup} objects.
	 */
	@NotNull
	@Override
	public List<WorkflowGroup> getWorkflowGroups(@NotNull final IDProvider element) {
		final Language previewLanguage = _context.requireSpecialist(WebeditUiAgent.TYPE).getPreviewLanguage();
		final Language uiLanguage = _context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage();

		final List<Dataset> allDatasets = new ArrayList<Dataset>();
		final List<WorkflowGroup> workflowGroups = new ArrayList<WorkflowGroup>();

		/**
		 * In WebClient 5, the context element (passed as a parameter) may be either a {@link PageRef} (on normal pages
		 * or in use with content projection) or a {@link Dataset} (on detail pages that show only a single dataset each).
		 */
		if (element instanceof PageRef) {
			final PageRef pageRef = ((PageRef) element);
			if (pageRef.getContent2Params() != null) {
				// Content projection is used; this indicates the context is a web page showing multiple datasets.
				final Content2Params content2Params = pageRef.getContent2Params();
				assert content2Params != null;
				final TableTemplate tableTemplate = content2Params.getTableTemplate();
				final List<Entity> entitiesInContentProjection = content2Params.getData(previewLanguage);
				for (final Entity entity : entitiesInContentProjection) {
					final Dataset dataset = tableTemplate.getDataset(entity);
					allDatasets.add(dataset);
				}
			} else if (pageRef.getPage() != null) {
				// Content projection is not used; the web page is a simple combination of PageRef and Page elements. Determine workflow groups based on the page reference and page objects alone.
				final Page page = pageRef.getPage();
				assert page != null;
				if (!pageRef.hasTask() && !page.hasTask()) {
					if ((pageRef.getReleaseStatus() != IDProvider.RELEASED && !pageRef.hasTask())) {
						final WorkflowGroup modifiedPage = Factory.create(_resourceBundle.getString(Group.MODIFIED.getDisplayName()), Collections.singletonList((IDProvider) pageRef));
						workflowGroups.add(modifiedPage);
					} else if ((page.getReleaseStatus() != IDProvider.RELEASED && !page.hasTask())) {
						final WorkflowGroup modifiedPage = Factory.create(_resourceBundle.getString(Group.MODIFIED.getDisplayName()), Collections.singletonList((IDProvider) page));
						workflowGroups.add(modifiedPage);
					} else {
						final WorkflowGroup releasedPage = Factory.create(_resourceBundle.getString(Group.RELEASED.getDisplayName()), Collections.singletonList((IDProvider) page));
						workflowGroups.add(releasedPage);
					}
				}
				if (pageRef.hasTask()) {
					final Task pageRefTask = pageRef.getTask();
					final Workflow pageRefWorkflow = (pageRefTask != null) ? pageRefTask.getWorkflow() : null;
					if (pageRefWorkflow != null) {
						final WorkflowGroup workflowedPageRef = Factory.create(pageRefWorkflow.getDisplayName(uiLanguage), Arrays.asList(pageRef, (IDProvider) page));
						workflowGroups.add(workflowedPageRef);
					}
				} else if (page.hasTask()) {
					final Task pageTask = page.getTask();
					final Workflow pageWorkflow = (pageTask != null) ? pageTask.getWorkflow() : null;
					if (pageWorkflow != null) {
						final WorkflowGroup workflowedPage = Factory.create(pageWorkflow.getDisplayName(uiLanguage), Collections.singletonList((IDProvider) page));
						workflowGroups.add(workflowedPage);
					}
				}
			}
		} else if (element instanceof Dataset) {
			// The WGP is operating in the context of a single, detail page of a dataset.
			final Dataset dataset = ((Dataset) element);
			allDatasets.add(dataset);
		} else {
			// The WGP is not operating within the context of a page reference or a dataset. In this case, it's unclear what to do, so it's best to do nothing.
			return Collections.emptyList();
		}

		final Map<Integer, List<IDProvider>> datasetsByReleaseState = sortDatasetsByReleaseState(allDatasets);
		if (!datasetsByReleaseState.get(-1).isEmpty()) {
			final Task datasetTask = datasetsByReleaseState.get(-1).get(0).getTask();
			final Workflow datasetWorkflow = (datasetTask != null) ? datasetTask.getWorkflow() : null;
			if (datasetWorkflow != null) {
				final WorkflowGroup workflowedDatasets = Factory.create(datasetWorkflow.getDisplayName(uiLanguage), datasetsByReleaseState.get(-1));
				workflowGroups.add(workflowedDatasets);
			}
		}
		if (!datasetsByReleaseState.get(IDProvider.CHANGED).isEmpty()) {
			final WorkflowGroup modifiedDatasets = Factory.create(_resourceBundle.getString(Group.MODIFIED.getDisplayName()), datasetsByReleaseState.get(IDProvider.CHANGED));
			workflowGroups.add(modifiedDatasets);
		}
		if (!datasetsByReleaseState.get(IDProvider.RELEASED).isEmpty()) {
			final WorkflowGroup releasedDatasets = Factory.create(_resourceBundle.getString(Group.RELEASED.getDisplayName()), datasetsByReleaseState.get(IDProvider.RELEASED));
			workflowGroups.add(releasedDatasets);
		}
		return workflowGroups;
	}


	/**
	 * This method is responsible for determining a page release status (released, changed, in workflow) and returning that
	 * status to the caller.
	 * <p/>
	 * The method itself builds upon {@code getWorkflowGroups(IDProvider element)}. The display names for the workflow
	 * groups containing "released" and "modified" elements are well-known. Thus, if {@code getWorkflowGroups(IDProvider
	 * element)} returns any other workflow groups with unknown names, assume that these contain elements that are
	 * currently in a workflow.
	 *
	 * @param element {@link PageRef} or {@link Dataset} object.
	 * @return The page release status for the {@code element}.
	 */
	@NotNull
	@Override
	public State getReleaseState(@NotNull final IDProvider element) {
		// Poll getWorkflowGroups(IDProvider element), using the store element object passed as a parameter.
		final List<WorkflowGroup> workflowGroups = getWorkflowGroups(element);

		Boolean encounteredModifiedGroup = false;
		Boolean encounteredUnknownGroup = false;

		// Iterate through the list of workflow groups.
		for (final WorkflowGroup workflowGroup : workflowGroups) {
			if (workflowGroup.getDisplayName().equals(_resourceBundle.getString(Group.MODIFIED.getDisplayName()))) {
				// If the workflow group has the well-known name (localized) used for modified elements, note this for later.
				encounteredModifiedGroup = true;
			} else if (!workflowGroup.getDisplayName().equals(_resourceBundle.getString(Group.RELEASED.getDisplayName()))) {
				// If the workflow group's display name does not match the localized, well-known names for released or modified elements, note this for later: it seems, there are groups with elements currently in workflows.
				encounteredUnknownGroup = true;
			}
		}

		if (encounteredUnknownGroup) {
			// If any groups with unknown names were encountered above, assume the page or some of its datasets are in workflows.
			return State.IN_WORKFLOW;
		}
		if (encounteredModifiedGroup) {
			// If a group's display name matched the well-known, localized name for modified elements, there are elements that have been modified; indicate this in the WebClient toolbar.
			return State.CHANGED;
		}
		// If no groups with workflowed or modified elements were encountered above, assume the page and its related elements are released.
		return State.RELEASED;
	}


	/**
	 * Based on all datasets collected by {@code getWorkflowGroups(IDProvider element)}, determine release and workflow
	 * status for each dataset and return these three lists as a map.
	 *
	 * @param datasets The list of datasets that should be inspected by this method.
	 * @return A map of dataset lists, separated by release/workflow status.
	 */
	@NotNull
	private Map<Integer, List<IDProvider>> sortDatasetsByReleaseState(@NotNull final List<Dataset> datasets) {
		final Map<Integer, List<IDProvider>> datasetsByReleaseState = new HashMap<Integer, List<IDProvider>>();
		final List<IDProvider> releasedDatasets = new ArrayList<IDProvider>();
		final List<IDProvider> modifiedDatasets = new ArrayList<IDProvider>();
		final List<IDProvider> workflowedDatasets = new ArrayList<IDProvider>();
		for (final Dataset dataset : datasets) {
			if (dataset.getReleaseStatus() == IDProvider.RELEASED && !dataset.hasTask()) {
				releasedDatasets.add(dataset);
			} else if (dataset.getReleaseStatus() != IDProvider.RELEASED && !dataset.hasTask()) {
				modifiedDatasets.add(dataset);
			} else if (dataset.hasTask()) {
				workflowedDatasets.add(dataset);
			}

		}

		datasetsByReleaseState.put(IDProvider.RELEASED, releasedDatasets);
		datasetsByReleaseState.put(IDProvider.CHANGED, modifiedDatasets);
		datasetsByReleaseState.put(-1, workflowedDatasets);
		return datasetsByReleaseState;
	}


}