package de.espirit.firstspirit.opt.example.webedit.elementstatusprovider;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.ReferenceEntry;
import de.espirit.firstspirit.access.Task;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.workflow.WebeditElementStatusProviderPlugin;
import de.espirit.firstspirit.workflow.WorkflowGroup;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;


public class StoreTypeElementStatusProviderPlugin implements WebeditElementStatusProviderPlugin {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.elementstatusprovider.ElementStatusProviderMessages";

	private Language _language;
	private ResourceBundle _resourceBundle;

	enum Group {
		ALL("StoreTypeElementStatusProviderPlugin.group.all"),
		PAGE("StoreTypeElementStatusProviderPlugin.group.page"),
		MEDIA("StoreTypeElementStatusProviderPlugin.group.media"),
		REMOTE("StoreTypeElementStatusProviderPlugin.group.remote");

		private final String _displayName;


		private Group(final String displayName) {
			_displayName = displayName;
		}
	}


	@Override
	public void setUp(@NotNull final BaseContext context) {
		if (_language == null) {
			_language = context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage();
		}
		if (_resourceBundle == null) {
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, _language.getLocale());
		}
	}


	@Override
	public void tearDown() {
	}


	@Override
	@NotNull
	public List<WorkflowGroup> getWorkflowGroups(@NotNull final IDProvider element) {
		final List<WorkflowGroup> result = new ArrayList<WorkflowGroup>();

		for (final Map.Entry<String, List<IDProvider>> entry : getGroups(element).entrySet()) {
			final List<IDProvider> elements = entry.getValue();
			if (!elements.isEmpty()) {
				final WorkflowGroup group = Factory.create(entry.getKey(), elements);
				result.add(group);
			}
		}
		return result;
	}


	/**
	 * Determine the page status shown in the WebClient. In this implementation, we'll consider only the context
	 * element, which should be a {@link PageRef} or a {@link de.espirit.firstspirit.access.store.contentstore.Dataset}
	 * (on content projection detail pages). If the context element has a workflow task, page status will be set to "In
	 * workflow". Otherwise, if the context element is modified, page status will be set to "Changed". Otherwise, page
	 * status will be set to "Released page".
	 *
	 * If the context element is a {@link PageRef}, the underlying {@link Page} will also be considered. The page status
	 * priority order given above still applies, but page status is set accordingly if either one of the {@link PageRef}/
	 * {@link Page} combination is in a given state.
	 * @param element The {@link IDProvider} upon which the WebClient preview is based.
	 * @return The {@link State} that should be displayed in the page status area of the WebClient toolbar.
	 */
	@Override
	@NotNull
	public State getReleaseState(@NotNull final IDProvider element) {
		// Assume the preview's web page is in a released state by default.
		State result = State.RELEASED;
		if (element instanceof PageRef) {
			// The context element is a PageRef. Process the underlying Page element first in a recursive call.
			final Page page = ((PageRef) element).getPage();
			if (page != null) {
				result = getReleaseState(page);
				if (result == State.IN_WORKFLOW) {
					// The Page element has a workflow task, so there is no need to process the PageRef.
					return State.IN_WORKFLOW;
				}
			}
		}
		// Process the state for the current store element.
		final State elementState = getState(element);
		if (elementState == State.IN_WORKFLOW) {
			// The element has a workflow task.
			return State.IN_WORKFLOW;
		} else if (elementState == State.CHANGED) {
			// The element is modified.
			result = State.CHANGED;
		}
		return result;
	}


	@NotNull
	private State getState(@NotNull final IDProvider element) {
		if (element.hasTask()) {
			return State.IN_WORKFLOW;
		}
		if (element.isReleaseSupported() && element.getReleaseStatus() != IDProvider.RELEASED) {
			return State.CHANGED;
		}
		return State.RELEASED;
	}


	@NotNull
	private Map<String, List<IDProvider>> getGroups(@NotNull final IDProvider element) {
		final Map<String, List<IDProvider>> groups = new HashMap<String, List<IDProvider>>();
		for (final Group group : Group.values()) {
			mapHandler(groups, getGroupName(group));
		}

		final Boolean elementHasTask = handleTaskElements(groups, element);
		collectOutgoingReferences(groups, element);

		if (element instanceof PageRef) {
			final Page page = ((PageRef) element).getPage();
			if (page != null) {
				collectOutgoingReferences(groups, page);
				for (final Section<?> section : page.getChildren(Section.class, true)) {
					collectOutgoingReferences(groups, section);
				}

				if (!handleTaskElements(groups, page)) {
					groups.get(getGroupName(Group.ALL)).add(page);
					groups.get(getGroupName(Group.PAGE)).add(page);
				}
			}

			if (!elementHasTask) groups.get(getGroupName(Group.PAGE)).add(element);
		}
		if (!elementHasTask) {
			groups.get(getGroupName(Group.ALL)).add(element);
		}
		if (groups.get(getGroupName(Group.ALL)).size() < 2 || groups.get(getGroupName(Group.ALL)).equals(groups.get(getGroupName(Group.PAGE)))) {
			groups.remove(getGroupName(Group.ALL));
		}
		return groups;
	}


	private void collectOutgoingReferences(@NotNull final Map<String, List<IDProvider>> groups, @NotNull final IDProvider element) {
		for (final ReferenceEntry entry : element.getOutgoingReferences()) {
			if (entry.isBroken()) {
				// This plug-in currently does not handle broken references.
			} else if (entry.isRemote()) {
				final IDProvider remoteElement = entry.getReferencedElement();
				if (remoteElement != null) {
					if (!handleTaskElements(groups, remoteElement)) {
						if (!groups.get(getGroupName(Group.REMOTE)).contains(remoteElement)) {
							groups.get(getGroupName(Group.REMOTE)).add(remoteElement);
						}
					}
				}
			} else if (entry.isType(ReferenceEntry.MEDIA_STORE_REFERENCE)) {
				final IDProvider referencedElement = entry.getReferencedElement();
				if (referencedElement != null) {
					if (!handleTaskElements(groups, referencedElement)) {
						if (!groups.get(getGroupName(Group.ALL)).contains(referencedElement)) {
							groups.get(getGroupName(Group.ALL)).add(referencedElement);
						}
						if (!groups.get(getGroupName(Group.MEDIA)).contains(referencedElement)) {
							groups.get(getGroupName(Group.MEDIA)).add(referencedElement);
						}
					}
				}
			}
		}
	}


	private void mapHandler(@NotNull final Map<String, List<IDProvider>> groups, @NotNull final String key) {
			if (groups.get(key) == null) {
				final List<IDProvider> emptyElementList = new ArrayList<IDProvider>();
				groups.put(key, emptyElementList);
			}
	}


	/**
	 * Handle elements with tasks by creating a group for each distinct task if it doesn't exist yet and adding the element to that group.
	 * @param groups The {@link List} of {@link IDProvider}s to work on.
	 * @param element The element to process.
	 * @return {@code true} if the element has a task, {@code false} otherwise.
	 */
	private Boolean handleTaskElements(@NotNull final Map<String, List<IDProvider>> groups, @NotNull final IDProvider element) {
		if (element.hasTask()) {
			final Task task = element.getTask();
			if (task != null) {
				mapHandler(groups, getGroupName(task));
				groups.get(getGroupName(task)).add(element);
				return true;
			}
		}
		return false;
	}


	@NotNull
	private String getGroupName(@NotNull final Group group) {
		return _resourceBundle.getString(group._displayName);
	}


	@NotNull
	String getGroupName(@NotNull final Task task) {
		return _resourceBundle.getString("StoreTypeElementStatusProviderPlugin.label.workflow") + ": " + task.getTaskState().getModelState().getDisplayName(_language);
	}
}
