package de.espirit.firstspirit.opt.example.webedit.inlineedit;

import de.espirit.common.tools.Streams;
import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.ElementDeletedException;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.LockException;
import de.espirit.firstspirit.access.store.pagestore.Body;
import de.espirit.firstspirit.access.store.pagestore.Content2Section;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.pagestore.SectionReference;
import de.espirit.firstspirit.agency.FormsAgent;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.client.EditorNode;
import de.espirit.firstspirit.forms.Form;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.forms.FormField;
import de.espirit.firstspirit.forms.InvalidFormDefinition;
import de.espirit.firstspirit.ui.operations.DisplayElementOperation;
import de.espirit.firstspirit.ui.operations.ShowFormDialogOperation;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.webedit.plugin.WebeditInlineEditItemsPlugin;
import de.espirit.firstspirit.webedit.plugin.inlineedit.ClientScriptProvidingInlineEditItem;
import de.espirit.firstspirit.webedit.plugin.inlineedit.ExecutableInlineEditItem;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditContext;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditItem;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.ResourceBundle;


/**
 * SectionLifespanWebeditInlineEditItemsPlugin is a first-tier class implementing a FirstSpirit WebClient 5 InlineEdit
 * plug-in. This class is responsible for collecting objects that define individual action items which will be shown as
 * InlineEdit buttons.
 */
public class SectionLifespanWebeditInlineEditItemsPlugin implements WebeditInlineEditItemsPlugin {

	private BaseContext _context;


	/**
	 * Set up persistence.
	 *
	 * @param context The context the plug-in operates in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
	}


	/**
	 * Perform clean-up before an instance of this class is destroyed.
	 */
	@Override
	public void tearDown() {
	}


	/**
	 * Assemble a list of object instances that should be represented as InlineEdit buttons in the user interface.
	 *
	 * @return A list of object instances based on an implementation of InlineEditItem. No two objects may be instances of
	 *         the same class. Return <code>Collections.emptyList()</code> if no suitable objects can be identified.
	 * @see ExecutableInlineEditItem
	 */
	@NotNull
	@Override
	public Collection<? extends InlineEditItem> getItems() {
		return Arrays.asList(new DefineLifespanExecutableInlineEditItem(_context), new ViewLifespanClientScriptProvidingInlineEditItem(_context));
	}


	/**
	 * DefineLifespanExecutableInlineEditItem creates a dialog that allows a user to set publication dates for a section.
	 * The user can select a start date and end date (or an open end date), which determines whether the section will be
	 * included in generated output of the page or not.
	 * <p/>
	 * This class implements an executable action, meaning it is able to use WebClient API agents to draw dialogs, work on
	 * forms and more.
	 */
	private static class DefineLifespanExecutableInlineEditItem implements ExecutableInlineEditItem {

		private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.inlineedit.InlineEditMessages";

		private static final String DEFINE_PERIOD = "define_period";
		private static final String VALIDFROM = "validfrom";
		private static final String VALIDTO = "validto";
		private static final String DEFINE_NO_END = "define_no_end";

		private final ResourceBundle _resourceBundle;

		private final String _formDefinition;
		private final String _rulesetDefinition;


		/**
		 * Constructor for this class: performs basic setup for persistence.
		 *
		 * @param context The context the plug-in lives in.
		 */
		DefineLifespanExecutableInlineEditItem(@NotNull final BaseContext context) {
			_formDefinition = readXml(context, "DefineLifespanFormDefinition.xml");
			_rulesetDefinition = readXml(context, "DefineLifespanRulesetDefinition.xml");
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		}


		/**
		 * Return the display name for button items based on this class.
		 *
		 * @param context The context to be considered for providing a label.
		 * @return A simple alphanumeric String that may contain spaces and some special characters.
		 */
		@NotNull
		@Override
		public String getLabel(@NotNull final InlineEditContext context) {
			return _resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.DefineLifespanExecutableInlineEditItem.button.tooltip");
		}


		/**
		 * Return the path and file name of the icon graphic used to illustrate a button item based on this class.
		 *
		 * @param context The context to be considered for providing the path.
		 * @return A String containing the path and file name of the icon graphic, relative to the root of a WebClient 5 web
		 *         application directory.
		 */
		@Nullable
		@Override
		public String getIconPath(@NotNull final InlineEditContext context) {
			return "webclient_examples/media/inlineedit/button_define-lifespan.png";
		}


		/**
		 * Determine whether the button item should be displayed in an InlineEdit button bar, given a specific context at
		 * run-time.
		 *
		 * @param context The context to be considered for deciding upon visibility.
		 * @return <code>true</code> if the button should be displayed, <code>false</code> if the button should not be
		 *         displayed.
		 */
		@Override
		public boolean isVisible(@NotNull final InlineEditContext context) {
			if (context.isMeta()) {
				return false;
			}
			final IDProvider element = context.getElement();
			return !(element instanceof Content2Section || element instanceof SectionReference) && element instanceof Section && context.isEditMode() && isElement(context);
		}


		/**
		 * Determine whether the button should be enabled (i.e., be available for interaction), given a specific context at
		 * run-time. If the button is disabled, its click action is not available, and its icon will be shown in weak
		 * contrast.
		 *
		 * @param context The context to be considered for deciding upon the item being enabled.
		 * @return <code>true</code> if the button should be enabled, <code>false</code> if the button should be disabled.
		 */
		@Override
		public boolean isEnabled(@NotNull final InlineEditContext context) {
			final IDProvider element = context.getElement();
			return element != null && (!element.isPermissionSupported() || element.getPermission().canChange());
		}


		/**
		 * Set up and display a form that allows a user to set publication periods on the context element (this method expects
		 * to be run in the context of a Page Store section object). Once form data is saved by the user, set the new
		 * publication settings on the element and save its associated page.
		 *
		 * @param context The context operating in.
		 */
		@Override
		public void execute(@NotNull final InlineEditContext context) {
			final Language language = context.getLanguage();
			final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);
			final ShowFormDialogOperation operation = operationAgent.getOperation(ShowFormDialogOperation.TYPE);
			if (operation == null) {
				throw new IllegalStateException("Attempted to obtain a ShowFormDialogOperation in an execution context where this operation is not available.");
			}
			try {
				final Section<?> section = (Section<?>) context.getElement();
				assert section != null;
				final Body body = (Body) section.getParent();
				assert body != null;
				final Page page = (Page) body.getParent();
				assert page != null;
				page.setLock(true, true);
				try {
					final Form form = context.requireSpecialist(FormsAgent.TYPE).getForm(_formDefinition);
					final FormData lifespanData = getLifespanData(section, form);
					operation.setTitle(section.getDisplayName(language) + ": " + _resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.DefineLifespanExecutableInlineEditItem.dialog.context"));
					operation.setContextElement(section);
					operation.setModified(true);
					operation.setFormData(lifespanData);
					operation.setRuleset(_rulesetDefinition);
					final FormData defaultSettings = form.createFormData();
					final FormField<?> defaultEndPeriodSetting = defaultSettings.get(null, DEFINE_NO_END);
					defaultEndPeriodSetting.set(true);
					operation.setDefaults(defaultSettings);
					final FormData formData = operation.perform(form, Collections.singletonList(language));
					if (formData != null) {
						setLifespanData(section, formData);
						page.save("Setting publication period.");
					}
				} catch (final InvalidFormDefinition e) {
					throw new IllegalArgumentException(e.getMessage());
				} catch (final ShowFormDialogOperation.InvalidRulesetDefinition e) {
					throw new IllegalArgumentException(e.getMessage());
				} finally {
					page.setLock(false, true);
				}
			} catch (final LockException e) {
				throw new IllegalArgumentException(e.getMessage());
			} catch (final ElementDeletedException e) {
				throw new IllegalArgumentException(e.getMessage());
			}
			final WebeditUiAgent webeditUiAgent = context.requestSpecialist(WebeditUiAgent.TYPE);
			if (webeditUiAgent != null) {
				final IDProvider previewElement = webeditUiAgent.getPreviewElement();
				final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
				if (displayElementOperation == null) {
					throw new IllegalStateException("Attempted to obtain a DisplayElementOperation in an execution context where this operation is not available.");
				}
				displayElementOperation.setLanguage(language);
				displayElementOperation.perform(previewElement);
			}
		}


		/**
		 * Fetch current publication period settings from a Section object and configure the form we will display to users
		 * accordingly.
		 *
		 * @param section The section element to fetch data from.
		 * @param form    The Form object to which we will attach the FormData we are assembling in this method.
		 * @return A FormData object that is attached to the Form object passed as a parameter.
		 * @throws InvalidFormDefinition
		 */
		private FormData getLifespanData(@NotNull final Section<?> section, @NotNull final Form form) throws InvalidFormDefinition {
			final FormData formData = form.createFormData();
			final long[] lifespan = section.getLifespan();
			formData.get(null, DEFINE_PERIOD).set(lifespan != null);
			if (lifespan != null && lifespan.length == 2) {
				formData.get(null, VALIDFROM).set(new Date(lifespan[0]));
				if (lifespan[1] != Section.OPEN_ENDED) {
					formData.get(null, DEFINE_NO_END).set(false);
					formData.get(null, VALIDTO).set(new Date(lifespan[1]));
				} else {
					formData.get(null, DEFINE_NO_END).set(true);
				}
			}
			return formData;
		}


		/**
		 * Handle new publication period settings submitted by the user (by saving the form dialog) and configure the Section
		 * object accordingly.
		 *
		 * @param section  The section element to store new publication settings to.
		 * @param formData The FormData object that contains new publication settings.
		 */
		private void setLifespanData(@NotNull final Section<?> section, @NotNull final FormData formData) {
			final Boolean definePeriod = (Boolean) formData.get(null, DEFINE_PERIOD).get();
			assert definePeriod != null;
			if (definePeriod) {
				final Date validFrom = (Date) formData.get(null, VALIDFROM).get();
				assert validFrom != null;
				Date validTo = (Date) formData.get(null, VALIDTO).get();
				if (validTo == null) {
					validTo = new Date(Calendar.DATE);
				}
				final Boolean defineEnd = (Boolean) formData.get(null, DEFINE_NO_END).get();
				assert defineEnd != null;
				if (defineEnd) {
					section.setLifespan(validFrom.getTime(), Section.OPEN_ENDED);
				} else {
					section.setLifespan(validFrom.getTime(), validTo.getTime());
				}
			} else {
				section.removeLifespan();
			}
		}


		/**
		 * Determine whether the element in whose context this button item runs is an EditorNode.
		 *
		 * @param context The context operating in.
		 * @return <code>true</code> if the context element is not an editor node or a sub-node of an editor node,
		 *         <code>false</code> if the context element is an editor node or a sub-node of one.
		 */
		private static boolean isElement(@NotNull final InlineEditContext context) {
			final String tag = context.getTag();
			if (!"CMS_MODULE".equals(tag)) {
				return false;
			}
			final EditorNode editorNode = context.getEditorNode();
			return editorNode == null || editorNode.getItemNode() == null;
		}


		/**
		 * Read an XML file located in the same package as this class and return its content as a String. Files are expected
		 * to be encoded in UTF-8.
		 *
		 * @param context      The context this button item is operating in. Only used for logging in case an error occurs.
		 * @param resourceName The file name.
		 * @return The content of the file.
		 */
		@Nullable
		private String readXml(@NotNull final BaseContext context, @NotNull final String resourceName) {
			final InputStream stream = getClass().getResourceAsStream(resourceName);
			try {
				return Streams.toString(stream, "UTF-8");
			} catch (final IOException e) {
				context.logError("Could not read XML file " + resourceName, e);
			}
			return null;
		}


	}

	/**
	 * ViewLifespanClientScriptProvidingInlineEditItem displays a message box with a synopsis of publication dates (start
	 * and end date) of a section. It it only active if publication has been restricted to specific dates; if the section
	 * is always published, the button will be inactive.
	 * <p/>
	 * This class implements a script action, which will be run client-side and may be used to display simple information
	 * (without the requirement for much interaction) to the user.
	 */
	private static class ViewLifespanClientScriptProvidingInlineEditItem implements ClientScriptProvidingInlineEditItem {

		private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.inlineedit.InlineEditMessages";

		private final ResourceBundle _resourceBundle;


		/**
		 * Constructor for this class: performs basic setup for persistence.
		 *
		 * @param context The context this button item operates in.
		 */
		ViewLifespanClientScriptProvidingInlineEditItem(@NotNull final BaseContext context) {
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		}


		/**
		 * Return the display name for button items based on this class.
		 *
		 * @param context The context to be considered for providing a label.
		 * @return A simple alphanumeric String that may contain spaces and some special characters.
		 */
		@NotNull
		@Override
		public String getLabel(@NotNull final InlineEditContext context) {
			if (hasLifespan(context.getElement())) {
				return _resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.button.tooltip.active");
			} else {
				return _resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.button.tooltip.inactive");
			}
		}


		/**
		 * Return the path and file name of the icon graphic used to illustrate a button item based on this class.
		 *
		 * @param context The context to be considered for providing the path.
		 * @return A String containing the path and file name of the icon graphic, relative to the root of a WebClient 5 web
		 *         application directory.
		 */
		@Nullable
		@Override
		public String getIconPath(@NotNull final InlineEditContext context) {
			return "webclient_examples/media/inlineedit/button_view-lifespan.png";
		}


		/**
		 * Assemble and return the JavaScript that should be run within the WebClient's user interface when this button item
		 * is clicked.
		 *
		 * @param context The context this button item is operating in.
		 * @return A String containing JavaScript.
		 */
		@Nullable
		@Override
		public String getScript(@NotNull final InlineEditContext context) {
			final IDProvider storeElement = context.getElement();
			final StringBuilder resultString = new StringBuilder();
			if (storeElement instanceof Section) {
				final long[] sectionLifespan = ((Section<?>) storeElement).getLifespan();
				if (sectionLifespan != null && sectionLifespan.length == 2) {
					final Date startDate = new Date(sectionLifespan[0]);
					resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.start.1"));
					resultString.append(' ');
					resultString.append(startDate);
					resultString.append(' ');
					resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.start.2"));
					resultString.append(' ');
					if (sectionLifespan[1] == Section.OPEN_ENDED) {
						resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.noend"));
					} else {
						final Date endDate = new Date(sectionLifespan[1]);
						resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.end"));
						resultString.append(' ');
						resultString.append(endDate);
					}
				} else {
					resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.noperiod"));
				}
			} else {
				resultString.append(_resourceBundle.getString("SectionLifespanWebeditInlineEditItemsPlugin.ViewLifespanClientScriptProvidingInlineEditItem.dialog.message.noend"));
			}
			resultString.append('.');
			return "top.WE_API.Common.showMessage(\"" + resultString + "\");";
		}


		/**
		 * Determine whether the button item should be displayed in an InlineEdit button bar, given a specific context at
		 * run-time.
		 *
		 * @param context The context to be considered for deciding upon visibility.
		 * @return <code>true</code> if the button should be displayed, <code>false</code> if the button should not be
		 *         displayed.
		 */
		@Override
		public boolean isVisible(@NotNull final InlineEditContext context) {
			if (context.isMeta()) {
				return false;
			}
			final IDProvider element = context.getElement();
			return !(element instanceof Content2Section || element instanceof SectionReference) && element instanceof Section && isElement(context);
		}


		/**
		 * Determine whether the button should be enabled (i.e., be available for interaction), given a specific context at
		 * run-time. If the button is disabled, its click action is not available, and its icon will be shown in weak
		 * contrast.
		 *
		 * @param context The context to be considered for deciding upon the item being enabled.
		 * @return <code>true</code> if the button should be enabled, <code>false</code> if the button should be disabled.
		 */
		@Override
		public boolean isEnabled(@NotNull final InlineEditContext context) {
			final IDProvider element = context.getElement();
			return element != null && (element instanceof Section) && hasLifespan(element);
		}


		/**
		 * Determine whether the element in whose context this button item runs is an EditorNode.
		 *
		 * @param context The context operating in.
		 * @return <code>true</code> if the context element is not an editor node or a sub-node of an editor node,
		 *         <code>false</code> if the context element is an editor node or a sub-node of one.
		 */
		private static boolean isElement(@NotNull final InlineEditContext context) {
			final String tag = context.getTag();
			if (!"CMS_MODULE".equals(tag)) {
				return false;
			}
			final EditorNode editorNode = context.getEditorNode();
			return editorNode == null || editorNode.getItemNode() == null;
		}


		/**
		 * Helper method to determine if an element is indeed a Page Store section and whether it carries specific publication
		 * period settings or not.
		 *
		 * @param element An IDProvider.
		 * @return <code>true</code> if the element object passed as a parameter is a Section object and does currently have
		 *         publication period settings, <code>false</code> if the element is not a Section object or does not have any
		 *         publication period restrictions.
		 */
		private static boolean hasLifespan(@Nullable final IDProvider element) {
			if (element instanceof Section) {
				final long[] sectionLifespan = ((Section<?>) element).getLifespan();
				if (sectionLifespan != null && sectionLifespan.length == 2) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}

	}
}
