package de.espirit.firstspirit.opt.example.webedit.inlineedit;

import de.espirit.common.tools.Strings;
import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.User;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.templatestore.Workflow;
import de.espirit.firstspirit.agency.StoreElementAgent;
import de.espirit.firstspirit.agency.UserAgent;
import de.espirit.firstspirit.client.EditorNode;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.webedit.plugin.WebeditInlineEditItemsPlugin;
import de.espirit.firstspirit.webedit.plugin.inlineedit.ExecutableInlineEditItem;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditContext;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditItem;
import de.espirit.firstspirit.workflow.WorkflowAgent;
import de.espirit.firstspirit.workflow.model.Transition;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;
import java.util.ResourceBundle;


/**
 * This class implements an InlineEdit plug-in that allows users to start a workflow on a single dataset.
 */
public class StartWorkflowWebeditInlineEditItemsPlugin implements WebeditInlineEditItemsPlugin {

	private BaseContext _context;
	private String _workflowUid;


	/**
	 * This setup method uses the Servlet API to acquire a parameter set in the web application's web.xml file. {@code
	 * webedit.action.workflow} should contain the UID of the workflow that the class {@link
	 * StartWorkflowExecutableInlineEditItem} should work with.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
		// This example works within the Mithras Energy demo project delivered with FirstSpirit 5 servers.
		// In order to use another workflow, modify the variable uid, providing the UID of the desired workflow.
		final String uid = "Content Freigabe";
		final IDProvider workflow = getWorkflow(uid);
		if (workflow != null) {
			_workflowUid = uid;
		}
	}


	@Nullable
	private IDProvider getWorkflow(final String uid) {
		if (Strings.isEmpty(uid)) {
			return null;
		}
		return _context.requireSpecialist(StoreElementAgent.TYPE).loadStoreElement(uid, Workflow.UID_TYPE, false);
	}


	@Override
	public void tearDown() {
	}


	@NotNull
	@Override
	public Collection<? extends InlineEditItem> getItems() {
		return Collections.singletonList(new StartWorkflowExecutableInlineEditItem(_context, _workflowUid));
	}


	/**
	 * This class implements an executable InlineEdit action that starts a workflow on a dataset. The workflow to be used
	 * is determined in {@link StartWorkflowWebeditInlineEditItemsPlugin}--see that class' documentation.
	 */
	private static class StartWorkflowExecutableInlineEditItem implements ExecutableInlineEditItem {

		private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.inlineedit.InlineEditMessages";

		private final ResourceBundle _resourceBundle;

		private final BaseContext _pluginContext;
		private final String _workflowUid;


		StartWorkflowExecutableInlineEditItem(final BaseContext context, final String workflowUid) {
			_pluginContext = context;
			_workflowUid = workflowUid;
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		}


		@NotNull
		@Override
		public String getLabel(@NotNull final InlineEditContext context) {
			return _resourceBundle.getString("StartWorkflowWebeditInlineEditItemsPlugin.StartWorkflowExecutableInlineEditItem.label");
		}


		@Override
		public String getIconPath(@NotNull final InlineEditContext context) {
			return "webclient_examples/media/inlineedit/button_workflow-action.png";
		}


		@Override
		public boolean isVisible(@NotNull final InlineEditContext context) {
			if (context.isMeta() || !context.isEditMode() || !isElement(context)) {
				return false;
			}
			final IDProvider element = context.getElement();
			if (element instanceof Dataset) {
				final Dataset dataset = (Dataset) element;
				return dataset.isWorkflowAllowed(getWorkflow(), context.requireSpecialist(UserAgent.TYPE).getUser()) && !dataset.hasTask();
			}
			return false;
		}


		@Override
		public boolean isEnabled(@NotNull final InlineEditContext context) {
			final IDProvider element = context.getElement();
			final User user = context.requireSpecialist(UserAgent.TYPE).getUser();
			final Workflow workflow = getWorkflow();
			return workflow != null && element != null && element.isWorkflowAllowed(getWorkflow(), user);
		}


		@Override
		public void execute(@NotNull final InlineEditContext context) {
			final Workflow workflow = getWorkflow();
			if (workflow == null) {
				throw new IllegalStateException("Workflow '" + _workflowUid + "' not found.");
			}
			final Dataset dataset = (Dataset) context.getElement();
			if (dataset == null) {
				throw new IllegalArgumentException("No element defined.");
			}
			if (dataset.hasTask()) {
				throw new IllegalStateException("Element already is in a workflow.");
			}
			try {
				final WorkflowAgent.WorkflowProcessContext processContext = context.requireSpecialist(WorkflowAgent.TYPE).startWorkflow(workflow, dataset);
				if ( ! processContext.isActivityProcessed()) {
					final Transition transition = processContext.showActionDialog();
					processContext.doTransition(transition);
				}
			} catch (final Exception e) {
				final Language language = context.requireSpecialist(WebeditUiAgent.TYPE).getPreviewLanguage();
				throw new IllegalArgumentException("No task could be assigned to " + dataset.getDisplayName(language) + '.');
			}
		}


		@Nullable
		private Workflow getWorkflow() {
			if (_workflowUid != null) {
				return (Workflow) _pluginContext.requireSpecialist(StoreElementAgent.TYPE).loadStoreElement(_workflowUid, Workflow.UID_TYPE, false);
			}
			return null;
		}


		/**
		 * Determine whether the element in whose context this button item runs is an EditorNode.
		 *
		 * @param context The context operating in.
		 * @return <code>true</code> if the context element is not an editor node or a sub-node of an editor node,
		 *         <code>false</code> if the context element is an editor node or a sub-node of one.
		 */
		private static boolean isElement(@NotNull final InlineEditContext context) {
			final String tag = context.getTag();
			if (!"CMS_MODULE".equals(tag)) {
				return false;
			}
			final EditorNode editorNode = context.getEditorNode();
			return editorNode == null || editorNode.getItemNode() == null;
		}


	}
}
