package de.espirit.firstspirit.opt.example.webedit.mediamanagement;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.mediastore.Media;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.forms.FormField;
import de.espirit.firstspirit.webedit.plugin.media.ClientScriptProvidingMediaManagementItem;
import de.espirit.firstspirit.webedit.plugin.media.MediaManagementContext;
import de.espirit.firstspirit.webedit.plugin.media.MediaManagementItem;
import de.espirit.firstspirit.webedit.plugin.media.MediaManagementItemsPlugin;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;


/**
 * Example implementation of {@link MediaManagementItemsPlugin}.
 *
 * Based on a {@link Media} element's geolocation information (stored in metadata), provide a button item which opens
 * Google Maps and places a marker at the latitude and longitude on the map.
 */
public class GeolocationMediaManagementItemsPlugin implements MediaManagementItemsPlugin {

	/**
	 * Provide {@link MediaManagementItem} objects.
	 *
	 * @return A {@link Collection} of {@link MediaManagementItem} objects.
	 */
	@NotNull
	@Override
	public Collection<? extends MediaManagementItem> getItems() {
		return Collections.singletonList(new OpenGMapsClientScriptProvidingMediaManagementItem());
	}


	/**
	 * Sets up any necessary functionality before {@link #getItems()} can be called.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		// Nothing to do.
	}


	/**
	 * Performs basic clean-up.
	 */
	@Override
	public void tearDown() {
		// Nothing to do.
	}


	/**
	 * Implementation of {@link ClientScriptProvidingMediaManagementItem}.
	 *
	 * This item is shown as a button on {@link Media} elements in ContentCreator's media management UI. On click, it opens
	 * a new browser tab and navigates to a Google Maps URL which places a marker on the map based on the latitude and longitude
	 * stored in the Media element's metadata.
	 */
	private static class OpenGMapsClientScriptProvidingMediaManagementItem implements ClientScriptProvidingMediaManagementItem {

		/**
		 * Determines if the button item should be enabled. The button will be set to "enabled" iff {@link #getCoordinatesFromMetadata(IDProvider)}
		 * does not return an empty {@link Map}, and the returned Map object contains the {@link String} keys "latitude" and "longitude".
		 *
		 * @param context The context to be considered for deciding upon visibility.
		 * @return {@code true} if the button item should be enabled, otherwise {@code false}.
		 */
		@Override
		public boolean isEnabled(@NotNull final MediaManagementContext context) {
			final Map<String, Double> geolocationMap = getCoordinatesFromMetadata(context.getElement());
			return !geolocationMap.isEmpty() && geolocationMap.containsKey("latitude") && geolocationMap.containsKey("longitude");
		}


		/**
		 * Determines if the button item should be visible. The button will only be visible if the contextual element is of type {@link Media}.
		 *
		 * @param context The context to be considered for deciding upon the item being enabled.
		 * @return {@code true} if the contextual element is of type {@link Media}, otherwise {@code false}.
		 */
		@Override
		public boolean isVisible(@NotNull final MediaManagementContext context) {
			final IDProvider element = context.getElement();
			return element instanceof Media;
		}


		/**
		 * Provide the path to the button's icon (relative to the project-specific ContentCreator web application path).
		 *
		 * @param context The context to be considered for providing the path.
		 * @return An icon path.
		 */
		@Nullable
		@Override
		public String getIconPath(@NotNull final MediaManagementContext context) {
			return "webclient_examples/media/mediamanagement/button_open_gmaps.png";
		}


		/**
		 * Provides the JavaScript which will be run if the button is clicked.
		 *
		 * This method builds a Google Maps URL which is configured to place a marker at the contextual element's latitude and longitude coordinates
		 * and opened in a new browser tab or window.
		 *
		 * NOTE that Google Maps URL formats (i.e. the set and data formats of GET parameters) may change over time. The URL format works as of May 2015.
		 *
		 * @param context The plug-in context.
		 * @return JavaScript code.
		 */
		@Nullable
		@Override
		public String getScript(@NotNull final MediaManagementContext context) {
			// Obtain a Map object from #getCoordinatesFromMetadata(IDProvider) so we can read out latitude and longitude values while assembling the URL.
			final Map<String, Double> geolocationMap = getCoordinatesFromMetadata(context.getElement());
			if (!geolocationMap.isEmpty() && geolocationMap.containsKey("latitude") && geolocationMap.containsKey("longitude")) {
				return "window.open(" + '"' + "https://maps.google.com/maps?q=loc:" + geolocationMap.get("latitude") + ',' + geolocationMap.get("longitude") + "&z=20&t=h&hl=en" + '"' + ", \"_blank\");";
			}
			// No valid geolocation data is available for the contextual element, so we won't return any JavaScript.
			return null;
		}


		/**
		 * Provides the button item's label which will be shown as a tool tip.
		 *
		 * @param context The context to be considered for providing a label.
		 * @return The button item's label.
		 */
		@NotNull
		@Override
		public String getLabel(@NotNull final MediaManagementContext context) {
			return "Show geolocation in Google Maps";
		}


		@NotNull
		private Map<String, Double> getCoordinatesFromMetadata(@Nullable final IDProvider element) {
			if (element instanceof Media && element.hasMeta()) {
				final FormData metadataFormData = element.getMetaFormData();
				if (metadataFormData != null) {
					// Obtain the metadata form fields with names "FS_0x0002" for latitude and "FS_0x0004" for longitude.
					final FormField<?> latitudeFormField = metadataFormData.get(null, "FS_0x0002");
					final FormField<?> longitudeFormField = metadataFormData.get(null, "FS_0x0004");
					// Sanity check: geolocation EXIF data is stored in the form fields as Double values.
					if (latitudeFormField.get() instanceof Double && longitudeFormField.get() instanceof Double) {
						final Map<String, Double> geolocationMap = new HashMap<String, Double>(2);
						geolocationMap.put("latitude", (Double) latitudeFormField.get());
						geolocationMap.put("longitude", (Double) longitudeFormField.get());
						return geolocationMap;
					}
				}
			}
			// No valid geolocation data is available for the given IDProvider, or the IDProvider is not a Media element. Return an empty Map object.
			return Collections.emptyMap();
		}
	}
}
