package de.espirit.firstspirit.opt.example.webedit.storemapping;

import de.espirit.firstspirit.access.AccessRuntimeException;
import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.User;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.ElementDeletedException;
import de.espirit.firstspirit.access.store.LockException;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.access.store.pagestore.PageFolder;
import de.espirit.firstspirit.access.store.pagestore.PageStoreRoot;
import de.espirit.firstspirit.access.store.sitestore.SiteStoreFolder;
import de.espirit.firstspirit.access.store.sitestore.SiteStoreRoot;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.store.WebeditStoreMappingPlugin;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;


/**
 * Place new and moved pages underneath the WebClient system folder by web site section, as determined by the top-level
 * Site Store folder.
 */
public class SiteSectionsStoreMappingPlugin implements WebeditStoreMappingPlugin {

	private BaseContext _context;


	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
	}


	@Override
	public void tearDown() {
	}


	/**
	 * Return a {@link PageFolder} object that corresponds to the {@link SiteStoreFolder} {@code folder}, or {@code null}
	 * if no corresponding folder exists in the Page Store.
	 *
	 * @param folder The Site Store folder for which to retrieve a corresponding Page Store folder.
	 * @return A {@link PageFolder}, or {@code null} if no suitable folder can be identified.
	 */
	@Override
	public PageFolder requestMappedFolder(@NotNull final SiteStoreFolder folder) {
		return getMappedFolder(folder, false);
	}


	/**
	 * Return a {@link PageFolder} object that corresponds to the {@link SiteStoreFolder} {@code folder}. If necessary,
	 * create a suitable new folder in the Page Store.
	 *
	 * @param folder The Site Store folder for which to retrieve a corresponding Page Store folder.
	 * @return A {@link PageFolder}.
	 */
	@NotNull
	@Override
	public PageFolder requireMappedFolder(@NotNull final SiteStoreFolder folder) {
		final PageFolder mappedFolder = getMappedFolder(folder, true);
		// mappedFolder should never be null. In the unlikely event that it is, return the WebClient system folder instead.
		return mappedFolder != null ? mappedFolder : getSystemFolder();
	}


	/**
	 * Identify a {@link PageFolder} corresponding to {@link SiteStoreFolder} {@code folder}. If {@code required} is {@code
	 * true}, create a new folder if no suitable one exists at the time of execution.
	 *
	 * @param folder   The Site Store folder for which to retrieve a corresponding Page Store folder.
	 * @param required Specifies whether this method may return {@code null} or not. If this parameter's value is {@code
	 *                 true} and no suitable Page Store folder can be retrieved, this method will create one.
	 * @return A {@link PageFolder}, or {@code null} if {@code required} is {@code false} and no suitable folder can be
	 *         identified in the Page Store.
	 */
	@Nullable
	private PageFolder getMappedFolder(final SiteStoreFolder folder, final boolean required) {
		final PageFolder targetFolder;
		final PageFolder systemFolder = getSystemFolder();
		if (folder != null && folder.getStore().getType() == Store.Type.SITESTORE) {
			if (folder instanceof SiteStoreRoot) {
				return systemFolder;
			}
			if (!(folder.getParentFolder() instanceof SiteStoreRoot)) {
				targetFolder = getMappedFolder(folder.getParentFolder(), required);
				return targetFolder;
			} else {
				for (final PageFolder childFolder : systemFolder.getChildren(PageFolder.class, false)) {
					if (folder.getDisplayName(folder.getProject().getMasterLanguage()).equals(childFolder.getDisplayName(folder.getProject().getMasterLanguage()))) {
						return childFolder;
					}
				}

				if (!required) {
					return null;
				} else if (!systemFolder.getPermission().canAppendFolder()) {
					throw new AccessRuntimeException("Missing append folder permission!");
				}
				final Map<Language, String> displayNames = new HashMap<Language, String>();
				for (final Language lang : folder.getStore().getProject().getEditorialLanguages()) {
					displayNames.put(lang, folder.getDisplayName(lang));
				}
				try {
					targetFolder = systemFolder.createPageFolder(folder.getUid(), displayNames, true);
					if (folder.hasPermissions()) {
						// adopt permissions from Site Store folder
						try {
							targetFolder.setLock(true, false);
							final User user = folder.getProject().getUserService().getUser();
							targetFolder.setPermission(user, folder.getPermission());
							targetFolder.save("Element stored using StoreMapping class " + this.getClass().toString());
						} finally {
							targetFolder.setLock(false, false);
						}
					}
					return targetFolder;
				} catch (ElementDeletedException e) {
					throw new IllegalStateException("ElementDeletedException: " + e);
				} catch (LockException e) {
					throw new IllegalStateException("LockException: " + e);
				}
			}
		}
		throw new IllegalArgumentException("Argument must be an object of type de.espirit.firstspirit.access.store.sitestore.SiteStoreFolder.");
	}


	/**
	 * Identify the WebClient system folder. This is a Page Store folder selected in the project-specific "WebClient
	 * Settings" in the Server and Project Configuration application.
	 *
	 * @return PageFolder The Page Store folder that is set as the WebClient system folder.
	 */
	@NotNull
	private PageFolder getSystemFolder() {
		final PageStoreRoot pageStoreRoot = (PageStoreRoot) _context.requireSpecialist(StoreAgent.TYPE).getStore(Store.Type.PAGESTORE, false);
		final Project project = pageStoreRoot.getProject();
		final PageFolder systemFolder = project.getWebeditSystemFolder();
		return systemFolder != null ? systemFolder : pageStoreRoot;
	}
}
