package de.espirit.firstspirit.opt.example.webedit.toolbar;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.TableTemplateProvider;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.agency.ClientUrlAgent;
import de.espirit.firstspirit.agency.FormsAgent;
import de.espirit.firstspirit.agency.LanguageAgent;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.client.plugin.toolbar.ToolbarContext;
import de.espirit.firstspirit.forms.Form;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.ui.operations.ShowFormDialogOperation;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.webedit.plugin.WebeditToolbarActionsItemsPlugin;
import de.espirit.firstspirit.webedit.plugin.toolbar.ClientScriptProvidingToolbarActionsItem;
import de.espirit.firstspirit.webedit.plugin.toolbar.ExecutableToolbarActionsItem;
import de.espirit.firstspirit.webedit.plugin.toolbar.WebeditToolbarItem;

import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.ResourceBundle;


/**
 * Plug-in to provide two menu items for use in the ContentCreator toolbar menu "Actions". These menu actions present URLs
 * for ContentCreator and SiteArchitect based on the web page currently visible in the ContentCreator preview.
 */
public class PageInfoToolbarActionsItemsPlugin implements WebeditToolbarActionsItemsPlugin {

	private BaseContext _baseContext;


	/**
	 * Return a list of objects that define individual menu items.
	 *
	 * @return A list of objects that define individual menu items. No two objects may be instances of the same class.
	 */
	@NotNull
	@Override
	public Collection<? extends WebeditToolbarItem> getItems() {
		return Arrays.asList(new PageUrlExecutableToolbarActionsItem(_baseContext), new PageUrlScriptToolbarActionsItem(_baseContext));
	}


	/**
	 * Setup method. Store the context this toolbar menu item plug-in operates in.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_baseContext = context;
	}


	/**
	 * Closure method. If necessary, destruct any objects that cannot be handled properly by the garbage collector.
	 */
	@Override
	public void tearDown() {
	}


	/**
	 * This class defines an executable toolbar menu item that displays a form dialog with multiple links to different
	 * ContentCreator and SiteArchitect views of the web page currently displayed in the preview.
	 */
	private static class PageUrlExecutableToolbarActionsItem implements ExecutableToolbarActionsItem {

		private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.toolbar.ToolbarMessages";

		private final BaseContext _baseContext;
		private final ResourceBundle _resourceBundle;


		/**
		 * Constructor. Store the plug-in context locally for persistence and configure a resource bundle for localization,
		 * based on the ContentCreator's current user interface locale.
		 *
		 * @param context The context the {@link WebeditToolbarActionsItemsPlugin} operates in.
		 */
		PageUrlExecutableToolbarActionsItem(@NotNull final BaseContext context) {
			_baseContext = context;
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		}


		/**
		 * Return the label of this menu item.
		 *
		 * @param context The item's operational context.
		 * @return A label text.
		 */
		@NotNull
		@Override
		public String getLabel(@NotNull final ToolbarContext context) {
			return _resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.title");
		}


		/**
		 * Return the absolute or relative URL to an icon image file. Currently not used in conjunction with ContentCreator menu items.
		 *
		 * @param context The item's operational context.
		 * @return The path to an icon image file.
		 */
		@Override
		public String getIconPath(@NotNull final ToolbarContext context) {
			return null;
		}


		/**
		 * The execution code for this toolbar menu item, executed when the menu item is clicked. This code collects several
		 * URLs to elements related to the web page currently displayed in the ContentCreator preview; these URLs are direct links
		 * to these elements in either the ContentCreator or the SiteArchitect. Once all URLs are collected, an appropriate form is
		 * shown in a ContentCreator dialog.
		 *
		 * @param context The item's operational context.
		 */
		@Override
		public void execute(@NotNull final ToolbarContext context) {
			final IDProvider storeElement = context.getElement();
			if (!(storeElement instanceof PageRef || storeElement instanceof Dataset)) {
				return;
			}

			final Project project = storeElement.getProject();
			final Language uiLanguage = context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage();

			final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);
			final ShowFormDialogOperation showFormDialogOperation = operationAgent.getOperation(ShowFormDialogOperation.TYPE);
			if (showFormDialogOperation == null) {
				return;
			}

			final FormsAgent formsAgent = context.requireSpecialist(FormsAgent.TYPE);

			final StringBuilder formDefinition = new StringBuilder();
			formDefinition.append("<CMS_MODULE>");
			formDefinition.append("<CMS_LABEL name=\"description\" bold=\"yes\">");
			formDefinition.append("<LANGINFOS>");
			formDefinition.append("<LANGINFO lang=\"*\" label=\"").append(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.description")).append("\"/>");
			formDefinition.append("</LANGINFOS>");
			formDefinition.append("</CMS_LABEL>");
			formDefinition.append("<CMS_GROUP tabs=\"top\">");

			final List<Language> projectLanguages = project.getLanguages();
			for (final Language projectLanguage : projectLanguages) {
				formDefinition.append("<CMS_GROUP>");
				formDefinition.append("<LANGINFOS>");
				formDefinition.append("<LANGINFO lang=\"*\" label=\"").append(projectLanguage.getDisplayName(uiLanguage)).append(" (").append(projectLanguage.getAbbreviation()).append(")\"/>");
				formDefinition.append("</LANGINFOS>");
				formDefinition.append("<CMS_INPUT_TEXT name=\"webclient_").append(projectLanguage.getAbbreviation()).append("\" editable=\"no\">");
				formDefinition.append("<LANGINFOS>");
				formDefinition.append("<LANGINFO lang=\"*\" label=\"ContentCreator\"/>");
				formDefinition.append("</LANGINFOS>");
				formDefinition.append("</CMS_INPUT_TEXT>");
				formDefinition.append("</CMS_GROUP>");
			}

			formDefinition.append("</CMS_GROUP>");

			formDefinition.append("<CMS_GROUP>");
			formDefinition.append("<LANGINFOS>");
			formDefinition.append("<LANGINFO lang=\"*\" label=\"SiteArchitect\"/>");
			formDefinition.append("</LANGINFOS>");
			if (storeElement instanceof Dataset) {
				formDefinition.append("<CMS_INPUT_TEXT name=\"javaclient_contentstore\" editable=\"no\">");
				formDefinition.append("<LANGINFOS>");
				formDefinition.append("<LANGINFO lang=\"*\" label=\"").append(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.label.content2")).append("\"/>");
				formDefinition.append("</LANGINFOS>");
				formDefinition.append("</CMS_INPUT_TEXT>");
				formDefinition.append("<CMS_GROUP>");
				formDefinition.append("<LANGINFOS>");
				formDefinition.append("<LANGINFO lang=\"*\" label=\"Preview Page\"/>");
				formDefinition.append("</LANGINFOS>");
			}
			formDefinition.append("<CMS_INPUT_TEXT name=\"javaclient_pagestore\" editable=\"no\">");
			formDefinition.append("<LANGINFOS>");
			formDefinition.append("<LANGINFO lang=\"*\" label=\"").append(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.label.pagecontent")).append("\"/>");
			formDefinition.append("</LANGINFOS>");
			formDefinition.append("</CMS_INPUT_TEXT>");
			formDefinition.append("<CMS_INPUT_TEXT name=\"javaclient_sitestore\" editable=\"no\">");
			formDefinition.append("<LANGINFOS>");
			formDefinition.append("<LANGINFO lang=\"*\" label=\"").append(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.label.menustructure")).append("\"/>");
			formDefinition.append("</LANGINFOS>");
			formDefinition.append("</CMS_INPUT_TEXT>");
			if (storeElement instanceof Dataset) {
				formDefinition.append("</CMS_GROUP>");
			}
			formDefinition.append("</CMS_GROUP>");

			formDefinition.append("</CMS_MODULE>");

			final Form urlForm = formsAgent.getForm(formDefinition.toString());
			final FormData urlData = urlForm.createFormData();

			for (final Language projectLanguage : projectLanguages) {
				urlData.get(null, "webclient_" + projectLanguage.getAbbreviation()).set(getWebEditUrl(storeElement, projectLanguage, uiLanguage));
			}

			if (storeElement instanceof PageRef) {
				urlData.get(null, "javaclient_sitestore").set(getJavaClientUrl(storeElement, uiLanguage));
				final Page page = ((PageRef) storeElement).getPage();
				if (page != null) {
					urlData.get(null, "javaclient_pagestore").set(getJavaClientUrl(page, uiLanguage));
				} else {
					urlData.get(null, "javaclient_pagestore").set(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.noPageAvailable"));
				}
			} else {
				urlData.get(null, "javaclient_contentstore").set(getJavaClientUrl(storeElement, uiLanguage));
				final TableTemplate datasetTableTemplate = ((TableTemplateProvider) storeElement).getTableTemplate();
				if (datasetTableTemplate != null) {
					final PageRef datasetPreviewPageRef = datasetTableTemplate.getPreviewPageRef();
					if (datasetPreviewPageRef != null) {
						urlData.get(null, "javaclient_sitestore").set(getJavaClientUrl(datasetPreviewPageRef, uiLanguage));
						final Page page = datasetPreviewPageRef.getPage();
						if (page != null) {
							urlData.get(null, "javaclient_pagestore").set(getJavaClientUrl(page, uiLanguage));
						} else {
							urlData.get(null, "javaclient_pagestore").set(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.noPageAvailable"));
						}
					} else {
						urlData.get(null, "javaclient_sitestore").set("No PageRef identifiable.");
					}
				} else {
					urlData.get(null, "javaclient_sitestore").set("No URL identifiable.");
					urlData.get(null, "javaclient_pagestore").set("No URL identifiable.");
				}
			}

			showFormDialogOperation.setTitle(_resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlExecutableToolbarActionsItem.title"));
			showFormDialogOperation.setContextElement(storeElement);
			showFormDialogOperation.setMultiLanguage(false);
			showFormDialogOperation.setFormData(urlData);
			try {
				final LanguageAgent languageAgent = context.requestSpecialist(LanguageAgent.TYPE);
				if (languageAgent != null) {
					showFormDialogOperation.perform(urlForm, Collections.singletonList(languageAgent.getMetaLanguage()));
				}
			} catch (final ShowFormDialogOperation.InvalidRulesetDefinition e) {
				context.logError("Assembling the URL form failed.");
			}
		}


		/**
		 * Determine if this menu item should be enabled or disabled, depending on the current context.
		 *
		 * @param context This item's current operational context.
		 * @return {@code true} if the menu item should be enabled, {@code false} if it should be disabled.
		 */
		@Override
		public boolean isEnabled(@NotNull final ToolbarContext context) {
			return context.getElement() instanceof PageRef || context.getElement() instanceof Dataset;
		}


		/**
		 * Determine if this menu item should be visible or not, depending on the current context.
		 *
		 * @param context This item's current operation context.
		 * @return {@code true} if the menu item should be shown in the menu, {@code false} if it should be hidden.
		 */
		@Override
		public boolean isVisible(@NotNull final ToolbarContext context) {
			return context.getElement() instanceof PageRef || context.getElement() instanceof Dataset;
		}


		/**
		 * Private helper method to determine a ContentCreator URL based on various criteria.
		 *
		 * @param storeElement    The store element for which a URL should be determined.
		 * @param projectLanguage The project language that the {@code storeElement} should be displayed in.
		 * @param uiLanguage      The user interface language that the ContentCreator should be opened in.
		 * @return A URL to a ContentCreator view of the {@code storeElement}.
		 */
		private String getWebEditUrl(@NotNull final IDProvider storeElement, @NotNull final Language projectLanguage, @NotNull final Language uiLanguage) {
			final ClientUrlAgent urlAgent = _baseContext.requireSpecialist(ClientUrlAgent.TYPE);
			final ClientUrlAgent.WebeditUrlBuilder webeditUrlBuilder = urlAgent.getBuilder(ClientUrlAgent.ClientType.WEBEDIT);

			final Project project = storeElement.getProject();
			if (project == null) {
				return "Could not get URL.";
			}
			webeditUrlBuilder.element(storeElement);
			webeditUrlBuilder.project(project);
			webeditUrlBuilder.language(projectLanguage);
			webeditUrlBuilder.locale(uiLanguage.getLocale());
			return webeditUrlBuilder.createUrl();
		}


		/**
		 * Private helper method to determine a SiteArchitect URL based on various criteria.
		 *
		 * @param storeElement The store element for which a URL should be determined.
		 * @param uiLanguage   The user interface language that the SiteArchitect should be opened in.
		 * @return A URL to a SiteArchitect view of the {@code storeElement}.
		 */
		private String getJavaClientUrl(@NotNull final IDProvider storeElement, @NotNull final Language uiLanguage) {
			final ClientUrlAgent urlAgent = _baseContext.requireSpecialist(ClientUrlAgent.TYPE);
			final ClientUrlAgent.JavaClientUrlBuilder javaclientUrlBuilder = urlAgent.getBuilder(ClientUrlAgent.ClientType.JAVACLIENT);

			final Project project = storeElement.getProject();
			if (project == null) {
				return "Could not get URL.";
			}
			javaclientUrlBuilder.element(storeElement);
			javaclientUrlBuilder.project(project);
			javaclientUrlBuilder.locale(uiLanguage.getLocale());
			return javaclientUrlBuilder.createUrl();
		}


	}


	/**
	 * This class defines a JavaScript toolbar menu item that displays a message box with a direct URL to a ContentCreator view
	 * configured with the current interface language/preview language/preview element combination.
	 */
	private static class PageUrlScriptToolbarActionsItem implements ClientScriptProvidingToolbarActionsItem {

		private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.toolbar.ToolbarMessages";

		private final ResourceBundle _resourceBundle;


		/**
		 * Constructor. Store the plug-in context locally for persistence and configure a resource bundle for localization,
		 * based on the ContentCreator's current user interface locale.
		 *
		 * @param context The context the {@link WebeditToolbarActionsItemsPlugin} operates in.
		 */
		PageUrlScriptToolbarActionsItem(@NotNull final BaseContext context) {
			_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		}


		/**
		 * Return the absolute or relative URL to an icon image file. Currently not used in conjunction with ContentCreator menu items.
		 *
		 * @param context The item's operational context.
		 * @return The path to an icon image file.
		 */
		@Override
		public String getIconPath(@NotNull final ToolbarContext context) {
			return null;
		}


		/**
		 * Return the label of this menu item.
		 *
		 * @param context The item's operational context.
		 * @return A label text.
		 */
		@NotNull
		@Override
		public String getLabel(@NotNull final ToolbarContext context) {
			return _resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlScriptToolbarActionsItem.title");
		}


		/**
		 * This method assembles and returns the JavaScript code as a simple {@link String} object; the code will be executed
		 * on the client side when the menu item is clicked. The JavaScript here draws a simple message box with the ContentCreator
		 * URL, preconfigued to show the current web page in the current project language.
		 *
		 * @param context The item's operational context.
		 * @return A {@link String} of JavaScript code.
		 */
		@Override
		public String getScript(final ToolbarContext context) {
			final String result;
			final ClientUrlAgent urlAgent = context.requireSpecialist(ClientUrlAgent.TYPE);
			final ClientUrlAgent.WebeditUrlBuilder webeditUrlBuilder = urlAgent.getBuilder(ClientUrlAgent.ClientType.WEBEDIT);
			final IDProvider currentElement = context.getElement();
			final Project currentProject;
			if (currentElement != null) {
				currentProject = currentElement.getProject();
				if (currentProject != null) {
					webeditUrlBuilder.project(currentProject);
				}
				webeditUrlBuilder.element(currentElement);
				webeditUrlBuilder.language(context.requireSpecialist(WebeditUiAgent.TYPE).getPreviewLanguage());
				webeditUrlBuilder.locale(context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
				result = webeditUrlBuilder.createUrl();
			} else {
				result = _resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlScriptToolbarActionsItem.errorMessage");
			}
			return "top.WE_API.Common.showMessage(\"" + _resourceBundle.getString("PageInfoToolbarActionsItemsPlugin.PageUrlScriptToolbarActionsItem.title") + ": " + result + "\" );";
		}


		/**
		 * Determine if this menu item should be enabled or disabled, depending on the current context.
		 *
		 * @param context This item's current operational context.
		 * @return {@code true} if the menu item should be enabled, {@code false} if it should be disabled.
		 */
		@Override
		public boolean isEnabled(@NotNull final ToolbarContext context) {
			return true;
		}


		/**
		 * Determine if this menu item should be visible or not, depending on the current context.
		 *
		 * @param context This item's current operation context.
		 * @return {@code true} if the menu item should be shown in the menu, {@code false} if it should be hidden.
		 */
		@Override
		public boolean isVisible(@NotNull final ToolbarContext context) {
			return true;
		}
	}

}
