package de.espirit.firstspirit.opt.example.webedit.translationhelp;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.ModuleAgent;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.contentstore.Dataset;
import de.espirit.firstspirit.access.store.pagestore.Content2Section;
import de.espirit.firstspirit.access.store.pagestore.DataProvider;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.Section;
import de.espirit.firstspirit.access.store.pagestore.SectionReference;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.module.descriptor.ComponentDescriptor;
import de.espirit.firstspirit.webedit.plugin.WebeditInlineEditItemsPlugin;
import de.espirit.firstspirit.webedit.plugin.inlineedit.ExecutableInlineEditItem;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditContext;
import de.espirit.firstspirit.webedit.plugin.inlineedit.InlineEditItem;
import de.espirit.firstspirit.webedit.plugin.translation.TranslationPlugin;
import de.espirit.firstspirit.webedit.server.TranslationOperation;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;


/**
 * This class provides access to ContentCreator's translation help by specifying a button item shown in highlights over
 * sections and other content types if the project is configured with at least two project languages. If any {@link TranslationPlugin}
 * implementations are available in the project-specific ContentCreator web application context, one of these will be chosen
 * to provide automatic translation services.
 */
public class StartTranslationInlineEditItemsPlugin implements WebeditInlineEditItemsPlugin {

	private Class<?> _translationPlugin;


	@NotNull
	@Override
	public Collection<? extends InlineEditItem> getItems() {
		return Collections.singletonList(new OpenTranslationUIExecutableInlineEditItem(_translationPlugin));
	}


	@Override
	public void setUp(@NotNull final BaseContext context) {
		final ModuleAgent moduleAgent = context.requestSpecialist(ModuleAgent.TYPE);
		if (moduleAgent != null) {
			// There may be several TranslationPlugin implementations available. For simplicity's sake (and because
			// the TranslationOperation can only be configured with at most one TranslationPlugin), we will determine
			// the first implementation available in the project's ContentCreator web application and keep this one in a field for later use.
			final Collection<ComponentDescriptor> translationPlugins = moduleAgent.getComponents(TranslationPlugin.class);
			for (final ComponentDescriptor componentDescriptor : translationPlugins) {
				try {
					// We need to determine if the translation plug-in is available in the ContentCreator web application.
					// This can be done by attempting to load the class. If successful, we'll store that class in a field
					// and stop stepping through the list of TranslationPlugin implementations.
					_translationPlugin = moduleAgent.getClassLoader().loadClass(componentDescriptor.getComponentClass());
					return;
				} catch (final ClassNotFoundException e) {
					// Class is not available in the web application context.
				}
			}
		}
		_translationPlugin = null;
	}


	@Override
	public void tearDown() {

	}


	/**
	 * This class specifies a button item which opens the translation help dialog and optionally makes a translation plug-in
	 * available in order to use automatic translation services of some sort.
	 */
	private static class OpenTranslationUIExecutableInlineEditItem implements ExecutableInlineEditItem {

		private final Class<?> _translationPlugin;


		OpenTranslationUIExecutableInlineEditItem(@Nullable final Class<?> translationPlugin) {
			_translationPlugin = translationPlugin;
		}


		@NotNull
		@Override
		public String getLabel(@NotNull final InlineEditContext context) {
			// If a TranslationPlugin is available, let users know by providing a specialized button tooltip.
			if (_translationPlugin != null) {
				return "Open translation help (Uses " + _translationPlugin.getSimpleName() + " for automatic translation)";
			}
			return "Open translation help";
		}


		@Nullable
		@Override
		public String getIconPath(@NotNull final InlineEditContext context) {
			return "media/view/translation/compare.png";
		}


		@Override
		public void execute(@NotNull final InlineEditContext context) {
			if (context.getElement() instanceof DataProvider) {
				final DataProvider element = (DataProvider) context.getElement();
				final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);
				final TranslationOperation translationOperation = operationAgent.getOperation(TranslationOperation.TYPE);
				if (translationOperation != null) {
					translationOperation.setElement(element);

					// We'll use the current preview language as the source language.
					translationOperation.setSourceLanguage(context.getLanguage());

					// We need to find a target language (in which translation occurs). For simplicity's sake, we'll
					// use the first project language we encounter which does not equal the source language.
					Language targetLanguage = null;
					for (final Language projectLanguage : element.getProject().getLanguages()) {
						if (!projectLanguage.equals(context.getLanguage())) {
							targetLanguage = projectLanguage;
							break;
						}
					}

					// In #isVisible(...), we determined if the project has at least two project languages. Thus, we can
					// assert here that the target language is not null.
					assert targetLanguage != null;
					translationOperation.setTargetLanguage(targetLanguage);

					// If a TranslationPlugin implementation is available in the current ContentCreator web application,
					// configure the TranslationOperation for use with that class.
					if (_translationPlugin != null) {
						translationOperation.setTranslationPlugin(_translationPlugin.getName(), false);
					}

					translationOperation.perform();
				}
			}
		}


		@Override
		public boolean isVisible(@NotNull final InlineEditContext context) {
			// Translation only makes sense if the project is configured with at least two project languages, and the
			// element with which this item is associated is a form-based element (an instance of DataProvider).
			boolean enoughProjectLanguages = false;
			if (context.getElement() instanceof DataProvider) {
				final DataProvider element = (DataProvider) context.getElement();
				final Project project = element.getProject();
				if (project.getLanguages().size() >=2) {
					enoughProjectLanguages = true;
				}
			}

			// Limit availability of translation help to sections, pages, and datasets. Otherwise, the button may be shown on
			// many other things which have editor IDs.
			final IDProvider element = context.getElement();
			return enoughProjectLanguages && context.getEditorNode() == null && ((element instanceof Section && !(element instanceof SectionReference) && !(element instanceof Content2Section)) || element instanceof Page || element instanceof Dataset);
		}


		@Override
		public boolean isEnabled(@NotNull final InlineEditContext context) {
			// The button is always enabled. Access to the button is limited by its visibility (see #isVisible(...) above).
			return true;
		}
	}
}
