package de.espirit.firstspirit.opt.example.webedit.workflow;

import de.espirit.common.util.Listable;
import de.espirit.firstspirit.access.ReferenceEntry;
import de.espirit.firstspirit.access.script.Executable;
import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.pagestore.Page;
import de.espirit.firstspirit.access.store.pagestore.PageFolder;
import de.espirit.firstspirit.access.store.sitestore.PageRef;
import de.espirit.firstspirit.access.store.sitestore.SiteStoreFolder;
import de.espirit.firstspirit.access.store.sitestore.SiteStoreRoot;
import de.espirit.firstspirit.access.store.templatestore.WorkflowScriptContext;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.ui.operations.RequestOperation;
import de.espirit.firstspirit.webedit.WebeditUiAgent;

import java.io.PrintWriter;
import java.io.Writer;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;


/**
 * Delete workflow script.
 * <ul>
 *     <li>Supports Datasets.</li>
 *     <li>In case of incoming references
 *         <ul>
 *             <li>reject deletion if the user is no project-admin</li>
 *             <li>ask project-admins</li>
 *         </ul>
 *     </li>
 *     <li>Deletes PageRef's parent folder if it's a start node without siblings.</li>
 *     <li>Delete PageRef's page.</li>
 * </ul>
 */
public class Delete implements Executable {

	//--- Executable ---//

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.workflow.WorkflowMessages";


	/**
	 * Starts deletion.
	 *
	 * @param context root context
	 * @return {@code true} if deletion was successful, {@code false} otherwise.
	 */
	@SuppressWarnings("UseOfSystemOutOrSystemErr")
	@Override
	public Object execute(final Map<String, Object> context) {
		return execute(context, new PrintWriter(System.out, true), new PrintWriter(System.err, true));
	}


	/**
	 * Starts deletion.
	 *
	 * @param context root context
	 * @return {@code true} if deletion was successful, {@code false} otherwise.
	 */
	@Override
	public Object execute(final Map<String, Object> context, final Writer out, final Writer err) {
		final Object scriptContext = context.get("context");
		if (scriptContext instanceof WorkflowScriptContext) {
			checkAndDelete((WorkflowScriptContext) scriptContext, collectDeleteElements((WorkflowScriptContext) scriptContext));
		}
		return false;
	}


	//--- private methods ---//


	/**
	 * Collects all elements which should be deleted by this workflow script. E.g. for a {@link PageRef} it returns also
	 * the associated {@link Page}.
	 *
	 * @param context script context.
	 * @return all elements which should be deleted.
	 */
	private Collection<IDProvider> collectDeleteElements(final WorkflowScriptContext context) {
		final IDProvider element = context.getElement();

		if (element instanceof PageRef) {
			// special behavior for page-refs => delete page, maybe delete start node folders
			final List<IDProvider> elementsToDelete = new ArrayList<IDProvider>();

			// Check if the page reference is the only child element of its parent folder. If so, we'll delete the entire PageRefFolder. If not, we'll only delete the page reference.
			final PageRef pageRef = (PageRef) element;
			elementsToDelete.add((isSingleStartNode(pageRef) ? pageRef.getParentFolder() : pageRef));

			final Page page = pageRef.getPage();
			if (page != null) {
				final PageFolder pageFolder = page.getParent() instanceof PageFolder ? (PageFolder) page.getParent() : null;
				final Listable<IDProvider> pageFolderChildren = (pageFolder != null) ? pageFolder.getChildren(IDProvider.class, false) : null;
				final Iterator<IDProvider> pageFolderChildrenIterator = (pageFolderChildren != null) ? pageFolderChildren.iterator() : null;
				// Using the iterator, check if the parent PageFolder has only one child element. If that is so, we can safely delete the PageFolder. If not, we'll delete only the page.
				if (pageFolderChildrenIterator != null && pageFolderChildrenIterator.hasNext()) {
					pageFolderChildrenIterator.next();
					elementsToDelete.add(pageFolderChildrenIterator.hasNext() ? page : pageFolder);
				}
			}

			return elementsToDelete;
		} else {
			// default behaviour => just delete the given element
			return Collections.singleton(element);                                    // return element
		}
	}


	/**
	 * Checks the given {@code deleting} elements for incoming references, maybe asks the user and deletes all elements.
	 *
	 * @param context  script context.
	 * @param deleting elements which should be deleted.
	 */
	private Boolean checkAndDelete(final WorkflowScriptContext context, final Collection<IDProvider> deleting) {
		final ResourceBundle resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
		final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);
		final RequestOperation requestOperation = operationAgent.getOperation(RequestOperation.TYPE);
		if (requestOperation == null) {
			context.logError("Could not display message box to confirm element deletion. Aborting deletion process.");
			return false;
		}
		requestOperation.setTitle(resourceBundle.getString("Delete.title"));
		// check for inbound references (i.e. if other elements reference an element we are trying to delete)
		if (isRequired(deleting)) {
			// Other elements that are not about to be deleted do reference an element we are trying to delete. Prompt user depending on whether that user is an administrator or a standard user.
			if (context.getProject().isAdmin(context.getUser())) {
				// Administrator-level users are permitted to delete elements with inbound references (thereby possibly creating broken references). Ask that user if he/she is willing to take the risk.
				requestOperation.addYes();
				final RequestOperation.Answer noAnswer = requestOperation.addNo();
				final MessageFormat messageFormat = new MessageFormat("");
				messageFormat.applyPattern(resourceBundle.getString("Delete.message.has_incoming_references"));
				final Object[] messageArguments = {context.getElement().getDisplayName(context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage())};
				final RequestOperation.Answer submittedAnswer = requestOperation.perform(messageFormat.format(messageArguments));
				if (noAnswer.equals(submittedAnswer)) {
					return false;
				}
			} else {
				// Non-administrators may not create broken references. In this case, we'll present them a message box indicating the reason for the failed deletion action.
				requestOperation.addOk();
				requestOperation.perform(resourceBundle.getString("Delete.message.deletion_not_allowed"));
				return false;
			}
		} else {
			// Elements we are deleting are not being referenced by other elements (that are not about to be deleted as well); we'll prompt the user just to be sure.
			requestOperation.addYes();
			final RequestOperation.Answer noAnswer = requestOperation.addNo();
			final MessageFormat messageFormat = new MessageFormat("");
			messageFormat.applyPattern(resourceBundle.getString("Delete.message.general_deletion"));
			final Object[] messageArguments = {context.getElement().getDisplayName(context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage())};
			final RequestOperation.Answer submittedAnswer = requestOperation.perform(messageFormat.format(messageArguments));
			if (noAnswer.equals(submittedAnswer)) {
				return false;
			}
		}

		// Delete all elements
		delete(context, deleting);
		return true;
	}


	/**
	 * Deletes the given elements.
	 *
	 * @param elements which should be deleted.
	 */
	private void delete(final WorkflowScriptContext context, final Collection<IDProvider> elements) {
		try {
			// lock all
			for (final IDProvider lock : elements) {
				lock.setLock(true, true);
			}

			// delete all
			for (final IDProvider delete : elements) {
				delete.delete();
			}
		} catch (final Exception ex) {
			// try to unlock elements in case of an exception
			for (final IDProvider locked : elements) {
				try {
					locked.setLock(false, true);
				} catch (final Exception e) {
					context.logError("Could not unlock element " + locked.getElementType() + ' ' + locked.getUid() + " while deleting.", e);
				}
			}
		}
	}


	/**
	 * Returns {@code true} if at least one given element has incoming references from outside this set of elements.
	 *
	 * @param deleting elements that should be deleted.
	 * @return {@code true} if at least one given element has incoming references from outside.
	 */
	private boolean isRequired(final Collection<IDProvider> deleting) {
		for (final IDProvider delete : deleting) {
			if (isRequired(delete, deleting)) {
				return true;
			}
			for (final IDProvider child : delete.getChildren(IDProvider.class, true)) {
				if (isRequired(child, deleting)) {
					return true;
				}
			}
		}
		return false;
	}


	/**
	 * Returns {@code true} if the given {@code element} has incoming references outside the given set of {@code deleting}
	 * elements.
	 *
	 * @param element  to check for incoming references.
	 * @param deleting elements that should be deleted and which references can be ignored.
	 * @return {@code true} if the given {@code element} has incoming references outside.
	 */
	private boolean isRequired(final IDProvider element, final Collection<IDProvider> deleting) {
		for (final ReferenceEntry usage : element.getIncomingReferences()) {
			if (!deleting.contains(usage.getReferencedElement())) {
				return true; // referenced element is not part of deleting objects => element is required
			}
		}
		return false;
	}


	/**
	 * Returns {@code true} if the given {@code pageRef} is a {@link PageRef#isStartNode() start-node} without any
	 * siblings.
	 *
	 * @param pageRef to check.
	 * @return {@code true} if the given {@code pageRef} is a start-node without any siblings.
	 */
	private boolean isSingleStartNode(final PageRef pageRef) {
		Boolean elementIsSingleChild = false;
		final SiteStoreFolder pageRefFolder = pageRef.getParentFolder();
		final Listable<IDProvider> pageRefFolderChildren = (pageRefFolder != null && !(pageRefFolder instanceof SiteStoreRoot)) ? pageRefFolder.getChildren(IDProvider.class, false) : null;
		final Iterator<IDProvider> pageRefFolderChildrenIterator = (pageRefFolderChildren != null) ? pageRefFolderChildren.iterator() : null;
		// Using the iterator, check if the parent PageRefFolder has only one child element. If that is so and the page reference we're working with is a start node, we can safely delete the PageRefFolder as well.
		if (pageRefFolderChildrenIterator != null && pageRefFolderChildrenIterator.hasNext()) {
			pageRefFolderChildrenIterator.next();
			elementIsSingleChild = !pageRefFolderChildrenIterator.hasNext();
		}
		return pageRef.isStartNode() && elementIsSingleChild;
	}
}
