/*
	JavaScript controller object for the Phone Number Web Gadget code example.
*/

window.PhoneNumberWebGadget = function(webHost, configuration) {

	var self = this;

	// See de.espirit.firstspirit.webedit.server.gadgets.WebPluginGadgetHost for documentation of this object's functions.
	self.webHost = webHost;

	self.configuration = configuration;
	self.editable = false;

	// Tasks to perform once the gadget object is loaded, including setting input fields to desired initial states, etc.
	this.onLoad = function() {
		// Event listener which will be called as input fields receive data changes.
		function onChange() {
			self.webHost.onModification();
		}

		// Basic setup of the three form input fields.

		self.international_code = webHost.getElementById("international_code");
		self.international_code.onkeyup = onChange;
		self.international_code.onchange = onChange;
		self.international_code.disabled = !self.editable;

		self.city_code = webHost.getElementById("city_code");
		self.city_code.onkeyup = onChange;
		self.city_code.onchange = onChange;
		self.city_code.disabled = !self.editable;

		self.local_number = webHost.getElementById("local_number");
		self.local_number.onkeyup = onChange;
		self.local_number.onchange = onChange;
		self.local_number.disabled = !self.editable;

		if (self.international_code) {
			// Treat the form input field "international_code" depending on usage needs.
			if (!self.configuration.offerInternationalCode) {
				// The form input field is not needed: hide it, but still configure a placeholder option with country code "0" for storage purposes.
				self.international_code.options[0] = new Option("Country Code", "0", false, true);
				self.international_code.style.display = "none";
				self.local_number.style.width = "75%";
			} else {
				// The form input field is needed: populate the select control with options for country codes.
				// Specifically, the following lines use data from the configuration object preset with data in
				// the Java class PhoneNumberWebGadget.

				// If no international code is required, add an "empty" option which indicates no country code is selected.
				if (!self.configuration.requireInternationalCode) {
					self.international_code.options[0] = new Option("Country Code", "0", false, true);
				}

				// Parse and use the JSON string stored under the configuration object's "internationalCodes" key to
				// build options for individual country codes configured in the input component instance's GOM XML.
				var countries = JSON.parse(self.configuration.internationalCodes);
				for (var i = 0; i < countries.length; i++) {
					var country = countries[i];
					var bestLangInfo;
					for (var j = 0; j < country.langinfos.length; j++) {
						var langInfo = country.langinfos[j];
						if (!bestLangInfo && langInfo.lang == "*") {
							bestLangInfo = langInfo;
						}
						if (langInfo.lang == self.configuration.formLanguage) {
							bestLangInfo = langInfo;
						}
					}

					var defaultSelectedStatus = false;
					if (self.configuration.defaultCountryCode && country.code == self.configuration.defaultCountryCode) {
						defaultSelectedStatus = true;
					}

					var countryOption = new Option(country.code + " - " + bestLangInfo.label, country.code, defaultSelectedStatus, false);
					self.international_code.options[self.international_code.length] = countryOption;

					bestLangInfo = null;
				}
			}
		}

		// Populate the form input fields with data if the input component already has a value when the form is loaded.
		if (self.value) {
			if (self.value['INTERNATIONAL']) {
				self.international_code.value = self.value['INTERNATIONAL'];
			} else if (self.configuration.requireInternationalCode && self.configuration.defaultCountryCode) {
				self.international_code.value = self.configuration.defaultCountryCode;
			} else {
				self.international_code.value = "0";
			}

			self.city_code.value = self.value['CITY'] || "";
			self.local_number.value = self.value['LOCAL'] || "";
		}
	};

	/*
		NOTE: the following functions implement aspects (see Online Documentation for FirstSpirit:
		"Plug-In Development > Universal Extensions > Input Components > ContentCreator > JavaScriptController (Client)")
		of the web gadget.

		In order to implement a specific gadget aspect, all of the aspect's required functions must be defined here.
	*/


	// Aspect ValueHolder: getter function.
	// Provide the current value object based on current data in the gadget's input fields.
	this.getValue = function() {
		var value = self.value || {};
		var returnValue = {
			'INTERNATIONAL': value['INTERNATIONAL'] || "0",
			'CITY': value['CITY'] || "",
			'LOCAL': value['LOCAL'] || ""
		};
		if (self.international_code) {
			returnValue['INTERNATIONAL'] = self.international_code.value;
		}
		if (self.city_code) {
			returnValue['CITY'] = self.city_code.value;
		}
		if (self.local_number) {
			returnValue['LOCAL'] = self.local_number.value;
		}
		return returnValue;
	};


	// Aspect ValueHolder: setter function.
	// Set the gadget's input fields' values according to the provided value object.
	this.setValue = function(value) {
		self.value = value;
		if (self.international_code) {
			self.international_code.value = value['INTERNATIONAL'] || "0";
		}
		if (self.city_code) {
			self.city_code.value = value['CITY'] || "";
		}
		if (self.local_number) {
			self.local_number.value = value['LOCAL'] || "";
		}
	};


	// Aspect ValueHolder: empty check function.
	// Indicate if the gadget's value is deemed empty.
	this.isEmpty = function() {
		return self.city_code && !self.city_code.value
				&& self.local_number && !self.local_number.value;
	};


	// Aspect Labelable: getter function.
	// Provide the gadget's label.
	this.getLabel = function() {
		return self.configuration.label;
	};


	// Aspect Editable: setter function.
	// Set input fields enabled or disabled to allow and deny data input, respectively.
	this.setEditable = function(enable) {
		self.editable = enable;
		if (self.international_code) {
			self.international_code.disabled = !enable;
		}
		if (self.city_code) {
			self.city_code.disabled = !enable;
		}
		if (self.local_number) {
			self.local_number.disabled = !enable;
		}
	};
};